@@ -3,15 +3,18 @@
 # when using sympa. Aliases can be added or removed in file --SENDMAIL_ALIASES--
 # To use a different script, you should edit the 'alias_manager' sympa.conf parameter
 
-# RCS Identication ; $Revision: 6556 $ ; $Date: 2010-06-21 18:01:01 +0200 (lun 21 jun 2010) $ 
+# RCS Identication ; $Revision: 10083 $ ; $Date: 2014-01-01 01:12:12 +0100 (mer. 01 janv. 2014) $ 
 
 # L. Marcotte has written a version of alias_manager.pl that is LDAP enabled
 # check the contrib. page for more information :
 # http://sympa.org/contrib.html
 
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,8 +27,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 $ENV{'PATH'} = '';
 
@@ -47,7 +49,7 @@ my $tmp_alias_file = $Conf{'tmpdir'}.'/sympa_aliases.'.time;
 
 
 my $default_domain;
-my $alias_wrapper       = Sympa::Constants::SBINDIR . '/aliaswrapper';
+my $alias_wrapper       = Sympa::Constants::LIBEXECDIR . '/sympa_newaliases-wrapper';
 my $lock_file           = Sympa::Constants::EXPLDIR . '/alias_manager.lock';
 my $path_to_queue       = Sympa::Constants::LIBEXECDIR . '/queue';
 my $path_to_bouncequeue = Sympa::Constants::LIBEXECDIR .'/bouncequeue';
@@ -63,7 +65,7 @@ if (($operation !~ /^(add|del)$/) || ($#ARGV < 2)) {
 $default_domain = $Conf{'domain'};
 
 my $alias_file;
-$alias_file = $Conf{'sendmail_aliases'} || Sympa::Constants::SENDMAIL_ALIASES;
+$alias_file = Conf::get_robot_conf($domain, 'sendmail_aliases') || Sympa::Constants::SENDMAIL_ALIASES;
 $alias_file = $file if ($file);
 
 unless (-w "$alias_file") {
@@ -122,10 +124,14 @@ if ($operation eq 'add') {
 
     ## Newaliases
     unless ($file) {
-	unless (system($alias_wrapper) == 0) {
-	    print STDERR "Failed to execute newaliases: $!\n";
-	    exit(6)
+	unless (system($alias_wrapper, "--domain=$domain") == 0) {
+	    if ($? == -1) {
+		print STDERR "Failed to execute newaliases: $!\n";
+	    } else {
+		printf STDERR "newaliases exited with status %d\n", ($? >> 8);
 	    }
+	    exit(6);
+	}
     }
 
     ## Unlock
@@ -151,11 +157,11 @@ if ($operation eq 'add') {
     my @deleted_lines;
     while (my $alias = <ALIAS>) {
 	my $left_side = '';
-	$left_side = $1 if ($alias =~ /^([^:]+):/);
+	$left_side = $1 if ($alias =~ /^([^\s:]+)[\s:]/);
 
 	my $to_be_deleted = 0;
 	foreach my $new_alias (@aliases) {
-	    next unless ($new_alias =~ /^([^:]+):/);
+	    next unless ($new_alias =~ /^([^\s:]+)[\s:]/);
 	    my $new_left_side = $1;
 	    
 	    if ($left_side eq  $new_left_side) {
@@ -193,9 +199,13 @@ if ($operation eq 'add') {
 
     ## Newaliases
     unless ($file) {
-	unless (system($alias_wrapper) == 0) {
-	    print STDERR "Failed to execute newaliases: $!\n";
-	exit (6);
+	unless (system($alias_wrapper, "--domain=$domain") == 0) {
+	    if ($? == -1) {
+		print STDERR "Failed to execute newaliases: $!\n";
+	    } else {
+		printf STDERR "newaliases exited with status %d\n", ($? >> 8);
+	    }
+	    exit(6);
 	}
     }
     ## Unlock
@@ -221,11 +231,11 @@ sub already_defined {
     while (my $alias = <ALIAS>) {
 	# skip comment
 	next if $alias =~ /^#/ ; 
-	$alias =~ /^([^:]+):/;
+	$alias =~ /^([^\s:]+)[\s:]/;
 	my $left_side = $1;
 	next unless ($left_side);
 	foreach (@aliases) {
-	    next unless ($_ =~ /^([^:]+):/); 
+	    next unless ($_ =~ /^([^\s:]+)[\s:]/); 
 	    my $new_left_side = $1;
 	    if ($left_side eq  $new_left_side) {
 		print STDERR "Alias already defined : $left_side\n";
