@@ -1,11 +1,14 @@
 #!/usr/bin/perl
 
 # archived.pl - This script does the web archives building for Sympa
-# RCS Identication ; $Revision: 7965 $ ; $Date: 2012-11-19 10:41:47 +0100 (lun 19 nov 2012) $ 
+# RCS Identication ; $Revision: 11009 $ ; $Date: 2014-06-18 09:54:30 +0200 (mer. 18 juin 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +21,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ## Options :  F         -> do not detach TTY
 ##         :  d		-> debug -d is equiv to -dF
@@ -74,11 +76,11 @@ if ($main::options{'debug'}) {
 $main::options{'foreground'} = 1 if ($main::options{'debug'});
 $main::options{'log_to_stderr'} = 1 if ($main::options{'debug'} || $main::options{'foreground'});
 
-$wwsympa_conf    = Sympa::Constants::WWSCONFIG;
-$sympa_conf_file = Sympa::Constants::CONFIG;
+my $wwsympa_conf    = Sympa::Constants::WWSCONFIG;
+my $sympa_conf_file = Sympa::Constants::CONFIG;
 
-$wwsconf = {};
-$adrlist = {};
+my $wwsconf = {};
+my $adrlist = {};
 
 # Load WWSympa configuration
 unless ($wwsconf = &wwslib::load_config($wwsympa_conf)) {
@@ -118,7 +120,8 @@ unless ($main::options{'debug'} || $main::options{'foreground'}) {
    open(STDERR, ">> /dev/null");
    open(STDOUT, ">> /dev/null");
    if (open(TTY, "/dev/tty")) {
-      ioctl(TTY, $TIOCNOTTY, 0);
+        # ioctl(TTY, &TIOCNOTTY, 0);
+        ioctl(TTY, 0x20007471, 0);	# XXX s/b &TIOCNOTTY
       close(TTY);
    }
    setpgrp(0, 0);
@@ -133,8 +136,8 @@ unless ($main::options{'debug'} || $main::options{'foreground'}) {
 my $options;
 $options->{'stderr_to_tty'} = 1 if ($main::options{'foreground'});
 
-## Create and write the pidfile
-&tools::write_pid($wwsconf->{'archived_pidfile'}, $$, $options);
+## Create and write the PID file
+tools::write_pid('archived', $$, $options);
 
 # setting log_level using conf unless it is set by calling option
 if ($main::options{'log_level'}) {
@@ -185,10 +188,10 @@ do_log('notice', "archived %s Started", Sympa::Constants::VERSION);
 
 ## Catch SIGTERM, in order to exit cleanly, whenever possible.
 $SIG{'TERM'} = 'sigterm';
-$end = 0;
+my $end = 0;
 
 
-$queue = $Conf{'queueoutgoing'};
+my $queue = $Conf{'queueoutgoing'};
 print "queue : $queue\n";
 
 #if (!chdir($queue)) {
@@ -217,17 +220,19 @@ while (!$end) {
        last if $end;
        next unless (-f "$queue/$file");
 
+        next if $file =~ /\.(lock|NFSLock)\z/; # Skip lock.
+
        if ($file  =~ /^\.remove\.((.*)\.(\d\d\d\d\-\d\d))\.\d+$/ ) {
 	   my $arclistdir = $1;
-	   my $listadress = $2;
+	   my $listname = $2;
 	   my $yyyymm = $3;
-	   my $arcpath = "$wwsconf->{'arc_path'}/$listadress/$yyyymm";
+	   my $arcpath = "$wwsconf->{'arc_path'}/$listname/$yyyymm";
 
-	   do_log('debug',"start remove process :listadress :'$listadress' arclistdir '$arclistdir' arcpath '$arcpath'  yyyymm '$yyyymm'");
+	   do_log('debug',"start remove process :listname :'$listname' arclistdir '$arclistdir' arcpath '$arcpath'  yyyymm '$yyyymm'");
 	  
 	   my $list;
-	   unless ($list = new List ($listadress)) {
-	       do_log('err',"remove : unknown list $listadress");
+	   unless ($list = new List ($listname)) {
+	       do_log('err',"remove : unknown list $listname");
 	       next;
 	   }
 
@@ -240,9 +245,9 @@ while (!$end) {
 
 	   my $email_regexp = &tools::get_regexp('email');
 
-	   foreach my $removeorder (<REMOVE>) { 
-	       unless($removeorder =~ /(.*)\|\|($email_regexp)/){
-		   do_log ('err',"Ignoring remove_order $removeorder not recognized format");   
+	   foreach my $order (<REMOVE>) { 
+	       unless($order =~ /(.*)\|\|($email_regexp)/){
+		   do_log ('err',"Ignoring remove_order $order not recognized format");   
 		   next;
 	       }
 	       my $msgid = $1;
@@ -256,14 +261,15 @@ while (!$end) {
 
 	       my $message ;
 	       unless ($message = &Archive::search_msgid("$arcpath/arctxt",$msgid)){
-		   do_log('err','No message with message-id %s found in %s',$msgid,$arcpath);
+		   do_log('err','No message with message-id %s found in %s/arctxt',$msgid,$arcpath);
 		   next;
 	       }
 
 	       unless ($list->am_i('privileged_owner',$sender)|| $list->am_i('owner',$sender)||$list->am_i('editor', $sender)||&List::is_listmaster($sender,$list->{'domain'} )){
 		   # if not list owner or list editor or listmaster,n check if sender of remove order is sender of the message to remove
 
-		   unless (my $new_message = new Message("$arcpath/arctxt/$message",'noxsympato')) {
+		    my $new_message;
+		    unless ($new_message = new Message("$arcpath/arctxt/$message",'noxsympato')) {
 		        do_log('err',"unable to load new message $arcpath/arctxt/$message");
 			next;
 		   }
@@ -320,7 +326,7 @@ while (!$end) {
 	       next;
 	   }
        }else{
-	   my ($yyyy, $mm, $dd, $min, $ss, $adrlist);
+	   my ($yyyy, $mm, $dd, $hh, $min, $ss, $adrlist);
 	   
 	   if ($file =~ /^(\d{4})-(\d{2})-(\d{2})-(\d{2})-(\d{2})-(\d{2})-(.*)$/) {
 	       ($yyyy, $mm, $dd, $hh, $min, $ss, $adrlist) = ($1, $2, $3, $4, $5, $6, $7);
@@ -375,7 +381,7 @@ while (!$end) {
     &mail::reaper;
 }
 do_log('notice', 'archived exited normally due to signal');
-&tools::remove_pid($wwsconf->{'archived_pidfile'}, $$);
+tools::remove_pid('archived', $$);
 
 exit(0);
 
@@ -465,6 +471,7 @@ Removes the message having the identifier $msgid from the list named $adrlist.
 sub remove {
     my $adrlist = shift;
     my $msgid = shift;
+    my $robot = shift;
 
     do_log ('debug2',"remove ($adrlist, $msgid)");
     my $arc ;
@@ -478,13 +485,13 @@ sub remove {
     unless(&Log::db_log({'robot' => $robot,'list' => $adrlist,'action' => 'remove','parameters' => $msgid.','.$adrlist,'target_email' => '','msg_id' => $msgid,'status' => 'succes','error_type' => '','user_email' =>'','client' => $ip,'daemon' => $daemon_name})) {
 	&do_log('error','archived::remove: unable to log event');
     }
-  
+    my $arcpath = $wwsconf->{'arc_path'};
     $arc =~ /^(\d{4})-(\d{2})$/ ;
     my $yyyy = $1 ;
     my $mm = $2 ;
     
     $msgid =~ s/\$/\\\$/g;
-    system "$wwsconf->{'mhonarc'}  -outdir $wwsconf->{'arc_path'}/$adrlist/$yyyy-$mm -rmm $msgid";
+    system "$wwsconf->{'mhonarc'}  -outdir $arcpath/$adrlist/$yyyy-$mm -rmm $msgid";
 }
 
 =pod 
@@ -555,8 +562,9 @@ sub rebuild {
     my $tag = &get_tag($listname);
 
     my $list = new List($listname, $hostname);
-
-    do_log('debug',"Rebuilding $adrlist archive ($2)");
+    my $robot = $list->{'domain'};
+    my $arcpath = $wwsconf->{'arc_path'};
+    Log::do_log('debug3', "Rebuilding %s archive (%s)", $adrlist, $2);
 
     my $mhonarc_ressources = &tools::get_filename('etc',{},'mhonarc-ressources.tt2',$list->{'domain'}, $list);
 
@@ -566,22 +574,23 @@ sub rebuild {
 	&unset_hidden_mode();
     }
 
-    &do_log('notice',"Rebuilding  $arc with M2H_ADDRESSMODIFYCODE : %s",$ENV{'M2H_ADDRESSMODIFYCODE'});
+    Log::do_log('notice', "Rebuilding %s with M2H_ADDRESSMODIFYCODE : %s",
+	$arc, $ENV{'M2H_ADDRESSMODIFYCODE'});
 
     if ($arc) {
-        &do_log('notice',"Rebuilding  $arc of $adrlist archive");
+        Log::do_log('notice',"Rebuilding %s of %s archive", $arc, $adrlist);
 	$arc =~ /^(\d{4})-(\d{2})$/ ;
 	my $yyyy = $1 ;
 	my $mm = $2 ;
 
 	# remove empty directory
-	my $arcdir = $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$yyyy.'-'.$mm ;
+	my $arcdir = $arcpath.'/'.$adrlist.'/'.$yyyy.'-'.$mm ;
 	my $arctxt = $arcdir.'/arctxt' ;
 	if (opendir (DIR,$arctxt)) {
 	    my @files = (grep(/^\d+$/,(readdir DIR )));
 	    close (DIR);
 	    if ($#files == -1) { 
-		do_log('notice', "Removing empty directory $arcdir");
+		Log::do_log('notice', "Removing empty directory %s", $arcdir);
 		&tools::remove_dir ($arcdir);
 		next ;	 
 	    } 
@@ -597,21 +606,22 @@ sub rebuild {
 	}
 
 	## Remove .mhonarc.db
-	unlink $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$yyyy.'-'.$mm.'/.mhonarc.db';
+	unlink $arcpath.'/'.$adrlist.'/'.$yyyy.'-'.$mm.'/.mhonarc.db';
 	
 	## Remove existing HTML files
-	opendir HTML, "$wwsconf->{'arc_path'}/$adrlist/$yyyy-$mm";
+	opendir HTML, "$arcpath/$adrlist/$yyyy-$mm";
 	
 	## Skip arctxt/ . and ..
 	foreach my $html_file (grep !/^arctxt$|^index$|\.+$/, readdir(HTML)) {
-	    unlink $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$yyyy.'-'.$mm.'/'.$html_file;
+	    unlink $arcpath.'/'.$adrlist.'/'.$yyyy.'-'.$mm.'/'.$html_file;
 	}	
 	closedir HTML;
 
-	my $arcs_dir = &clean_archive_directory({'arc_root' => $wwsconf->{'arc_path'},'dir_to_rebuild' => "/$adrlist/$arc/arctxt"});
+	my $arcs_dir = Archive::clean_archive_directory(
+	    $robot, "/$adrlist/$arc/arctxt");
 	my $dir_to_rebuild = $arcs_dir->{'dir_to_rebuild'} if($arcs_dir);
 
-	my $cmd = "$wwsconf->{'mhonarc'} -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\' -rcfile $mhonarc_ressources -outdir $wwsconf->{'arc_path'}/$adrlist/$yyyy-$mm  -definevars \"listname='$listname' hostname=$hostname yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$wwsconf->{'arc_path'} base=$Conf{'wwsympa_url'}/arc tag=$tag\" -umask $Conf{'umask'} $dir_to_rebuild";
+	my $cmd = "$wwsconf->{'mhonarc'} -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\' -rcfile $mhonarc_ressources -outdir $arcpath/$adrlist/$yyyy-$mm  -definevars \"listname='$listname' hostname=$hostname yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$arcpath base=" . Conf::get_robot_conf($robot, 'wwsympa_url') . "/arc tag=$tag\" -umask $Conf{'umask'} $dir_to_rebuild";
 
 	do_log('debug',"System call : $cmd");
 	my $exitcode = system($cmd);
@@ -624,8 +634,8 @@ sub rebuild {
 	
 	## Remove lock if required
 	if ($exitcode == 75) {
-	    &do_log('notice', 'Removing lock directory %s', $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$arc.'/.mhonarc.lck');
-	    rmdir $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$arc.'/.mhonarc.lck';
+	    &do_log('notice', 'Removing lock directory %s', $arcpath.'/'.$adrlist.'/'.$arc.'/.mhonarc.lck');
+	    rmdir $arcpath.'/'.$adrlist.'/'.$arc.'/.mhonarc.lck';
 
 	    $exitcode = system($cmd);
 	    $exitcode = $exitcode / 256;	    
@@ -637,8 +647,8 @@ sub rebuild {
     }else{
         do_log('notice',"Rebuilding $adrlist archive completely");
 
-	if (!opendir(DIR, "$wwsconf->{'arc_path'}/$adrlist" )) {
-	    do_log('err',"unable to open $wwsconf->{'arc_path'}/$adrlist to rebuild archive");
+	if (!opendir(DIR, "$arcpath/$adrlist" )) {
+	    do_log('err',"unable to open $arcpath/$adrlist to rebuild archive");
 	    return ;
 	}
 	my @archives = (grep (/^\d{4}-\d{2}/, readdir(DIR)));
@@ -649,7 +659,7 @@ sub rebuild {
 	    my $yyyy = $1 ;
 	    my $mm = $2 ;
 
-	    my $arcdir = $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$yyyy.'-'.$mm;
+	    my $arcdir = $arcpath.'/'.$adrlist.'/'.$yyyy.'-'.$mm;
 
 	    ## Remove .mhonarc.db
 	    unlink $arcdir.'/.mhonarc.db';
@@ -661,8 +671,9 @@ sub rebuild {
 		unlink $arcdir.'/'.$html_file;
 	    }	
 	    closedir HTML;
-	    my $dir_to_rebuild = "$wwsconf->{'arc_path'}/$adrlist/$arc/arctxt";
-	    my $arcs_dir = &clean_archive_directory({'arc_root' => $wwsconf->{'arc_path'},'dir_to_rebuild' => "/$adrlist/$arc/arctxt"});
+	    my $dir_to_rebuild = $arcpath . "/$adrlist/$arc/arctxt";
+	    my $arcs_dir = Archive::clean_archive_directory(
+		$robot, "/$adrlist/$arc/arctxt");
 	    if($arcs_dir) {
 		$dir_to_rebuild = $arcs_dir->{'dir_to_rebuild'};
 	    }
@@ -671,7 +682,7 @@ sub rebuild {
 		&create_idx($arcdir);
 	    }
 
-	    my $cmd = "$wwsconf->{'mhonarc'} -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\'  -rcfile $mhonarc_ressources -outdir $arcdir  -definevars \"listname=$listname hostname=$hostname yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$wwsconf->{'arc_path'} base=$Conf{'wwsympa_url'}/arc tag=$tag\" -umask $Conf{'umask'} $dir_to_rebuild";
+	    my $cmd = "$wwsconf->{'mhonarc'} -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\'  -rcfile $mhonarc_ressources -outdir $arcdir  -definevars \"listname=$listname hostname=$hostname yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$arcpath base=" . Conf::get_robot_conf($robot, 'wwsympa_url') . "/arc tag=$tag\" -umask $Conf{'umask'} $dir_to_rebuild";
 	    my $exitcode = system($cmd);
 	    $exitcode = $exitcode / 256;
 
@@ -699,7 +710,7 @@ sub rebuild {
 
 =pod 
 
-=head2 sub mail2arc(STRING $file,STRING $listname,STRING $hostname,STRING $yyyy,STRING $mm,STRING $dd,STRING $hh,STRING $min,STRING $ss)
+=head2 sub mail2arc(STRING $file,STRING $listname,STRING $robot,STRING $yyyy,STRING $mm,STRING $dd,STRING $hh,STRING $min,STRING $ss)
 
 Archives one message into one list archives directory.
 
@@ -711,7 +722,7 @@ Archives one message into one list archives directory.
 
 =item * I<$listname>: a character string containing the name of the list in which to archive the message
 
-=item * I<$hostname>: a character string containing the name of the virtual robot hosting the list.
+=item * I<$robot>: a character string containing the name of the virtual robot hosting the list.
 
 =item * I<$yyyy>: a character string containing the year of the date when the message is archived (i.e. now)
 
@@ -767,14 +778,14 @@ Archives one message into one list archives directory.
 
 sub mail2arc {
 
-    my ($file, $listname, $hostname, $yyyy, $mm, $dd, $hh, $min, $ss) = @_;
+    my ($file, $listname, $robot, $yyyy, $mm, $dd, $hh, $min, $ss) = @_;
     my $arcpath = $wwsconf->{'arc_path'};
     my $newfile;
 
-    my $list = new List($listname, $hostname);
+    my $list = new List($listname, $robot);
 
     unless (defined $list) {
-	&do_log('err', 'Unknown list %s@%s', $listname, $hostname);
+	&do_log('err', 'Unknown list %s@%s', $listname, $robot);
 	return undef;
     }
 
@@ -790,7 +801,7 @@ sub mail2arc {
     #    chdir($wwsconf->{'arc_path'});
 
     if ($wwsconf->{'custom_archiver'}) {
-	`$wwsconf->{'custom_archiver'} --list=$listname\@$hostname --file=$queue/$file`;
+	`$wwsconf->{'custom_archiver'} --list=$listname\@$robot --file=$queue/$file`;
 	return 1;
     }else{
 
@@ -799,7 +810,7 @@ sub mail2arc {
 	if (! -d $basedir) {
 	    unless (mkdir $basedir, 0775) {
 		&do_log('err', 'Cannot create directory %s', $basedir);
-		unless (&List::send_notify_to_listmaster('unable_to_create_dir',$hostname,{'dir' => "$basedir"})) {
+		unless (&List::send_notify_to_listmaster('unable_to_create_dir',$robot,{'dir' => "$basedir"})) {
 		    &do_log('notice',"Unable to send notify 'unable_to_create_dir' to listmaster");
 		}
 	    }
@@ -862,35 +873,49 @@ sub mail2arc {
 	}
 	
 	## copy the file in the arctxt and in "mhonarc -add"
-	if( -f $monthdir."/index" )
-	{
+	if( -f $monthdir."/index" ){
 	    open(IDX,"<$monthdir/index") || fatal_err("couldn't read index for $listname");
 	    $newfile = <IDX>;
 	    chomp($newfile);
 	    $newfile++;
 	    close IDX;
-	}
-	else
-	{
+	}else{
 	    ## recreate index file if needed and update it
-	    $newfile = $index = &create_idx($monthdir) + 1;
+	    $newfile = &create_idx($monthdir) + 1;
 	}
-	
-	my $file_to_archive = $queue.'/'.$file;
-	my $cleaned_file = clean_archived_message({'path' => $queue, 'file' => $file});
-	$file_to_archive = $cleaned_file if($cleaned_file);
+
+	# savee arctxt dump of original message.
+	my $output = $Conf::Conf{'tmpdir'}.'/arc'.$newfile;
+	open ORIG, '<', "$queue/$file" or fatal_err("couldn't open file $queue/$file");
+	open DUMP, '>', "$arctxtdir/$newfile" or fatal_err("couldn't open file $arctxtdir/$newfile");
+	print DUMP join('', <ORIG>);
+	close ORIG;
+	close DUMP;
+
+	# prepare a temporary file with clean message content (htlm parts are cleaned) 
+	my $safe = Archive::clean_archived_message(
+	    $robot, "$arctxtdir/$newfile", $output);
+	unless ($safe) {
+	    do_log('err',"Could not clean message, ignoring message");
+	    next;
+	}
+
 	my $mhonarc_ressources = &tools::get_filename('etc',{},'mhonarc-ressources.tt2',$list->{'domain'}, $list);
 	
 	do_log ('debug',"calling $wwsconf->{'mhonarc'} for list %s", $list->get_list_id() ) ;
-	my $cmd = "$wwsconf->{'mhonarc'} -add -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\'  -rcfile $mhonarc_ressources -outdir $monthdir  -definevars \"listname='$listname' hostname=$hostname yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$wwsconf->{'arc_path'} base=$Conf{'wwsympa_url'}/arc tag=$tag\" -umask $Conf{'umask'} < $file_to_archive";
+
+	# call mhonarc on cleaned message source to make clean htlm view of message
+	my $cmd = "$wwsconf->{'mhonarc'} -add -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\'  -rcfile $mhonarc_ressources -outdir $monthdir  -definevars \"listname='$listname' hostname=$robot yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$arcpath base=" . Conf::get_robot_conf($robot, 'wwsympa_url') . "/arc tag=$tag\" -umask $Conf{'umask'} < $output";
 	
-	do_log('debug',"System call : $cmd");
+	do_log('debug',"System call : %s",$cmd);
 	
 	my $exitcode = system($cmd);
 	$exitcode = $exitcode / 256;
 	
 	# Delete temporary file containing escaped HTML.
-	if ($cleaned_file && -f $cleaned_file) {unlink $cleaned_file;}
+	if ($output && -f $output) {
+	    unlink $output;
+	}
 
 	## Remove lock if required
 	if ($exitcode == 75) {
@@ -903,16 +928,6 @@ sub mail2arc {
 	if ($exitcode) {
 	    do_log('err',"Command $cmd failed with exit code $exitcode");
 	}
-	
-	
-	open (ORIG, "$queue/$file") || fatal_err("couldn't open file $queue/$file");
-	open (DEST, ">$arctxtdir/$newfile") || fatal_err("couldn't open file $newfile");
-	while (<ORIG>) {
-	    print DEST $_ ;
-	}
-	
-	close ORIG;  
-	close DEST;
 
 	&save_idx("$monthdir/index",$newfile);
     }
@@ -953,7 +968,7 @@ Sets the value of $ENV{'M2H_ADDRESSMODIFYCODE'} and $ENV{'M2H_MODIFYBODYADDRESSE
 sub set_hidden_mode {
     my $tag = shift; ## tag is used as variable elements in tags to prevent message contents to be parsed
 
-    ## $ENV{'M2H_MODIFYBODYADDRESSES'}  positionner si le corps du message est parse
+    ## $ENV{'M2H_MODIFYBODYADDRESSES'} à positionner si le corps du message est parse
     $ENV{'M2H_ADDRESSMODIFYCODE'} = "s|^([^\@]+)\@([^\@]+)\$|\($tag\%hidden_head\%$tag\)\$1\($tag\%hidden_at\%$tag\)\$2\($tag\%hidden_end\%$tag\)|g";
     $ENV{'M2H_MODIFYBODYADDRESSES'} = 1;
 }
@@ -1106,66 +1121,6 @@ sub get_tag {
     return (substr(Digest::MD5::md5_hex(join('/', $Conf{'cookie'}, $listname)), -10)) ;
 }
 
-sub clean_archive_directory{
-    my $params = shift;
-    &do_log('debug',"Cleaning archives for directory '%s'.",$params->{'arc_root'}.'/'.$params->{'dir_to_rebuild'});
-    my $answer;
-    $answer->{'dir_to_rebuild'} = $params->{'arc_root'}.'/'.$params->{'dir_to_rebuild'};
-    $answer->{'cleaned_dir'} = $Conf::Conf{'tmpdir'}.'/'.$params->{'dir_to_rebuild'};
-    unless(my $number_of_copies = &tools::copy_dir($answer->{'dir_to_rebuild'},$answer->{'cleaned_dir'})){
-	&do_log('err',"Unable to create a temporary directory where to store files for HTML escaping (%s). Cancelling.",$number_of_copies);
-	return undef;
-    }
-    if(opendir ARCDIR,$answer->{'cleaned_dir'}){
-	my $files_left_uncleaned = 0;
-	foreach my $file (readdir(ARCDIR)){
-	    next if($file =~ /^\./);
-	    $files_left_uncleaned++ unless(&clean_archived_message({'path' => $answer->{'cleaned_dir'}, 'file' => $file, 'overwrite' => 1}));
-	}
-	closedir DIR;
-	if ($files_left_uncleaned) {
-	    &do_log('err',"HTML cleaning failed for %s files in the directory %s.",$files_left_uncleaned,$answer->{'dir_to_rebuild'});
-	}
-	$answer->{'dir_to_rebuild'} = $answer->{'cleaned_dir'};
-    }else{
-	&do_log('err','Unable to open directory %s: %s',$answer->{'dir_to_rebuild'},$!);
-	&tools::del_dir($answer->{'cleaned_dir'});
-	return undef;
-    }
-    return $answer;
-}
-
-sub clean_archived_message{
-    my $params = shift;
-    &do_log('debug',"Cleaning HTML in archived file '%s/%s' (overwrite: %s).",$params->{'path'},$params->{'file'},$params->{'overwrite'});
-    my $filtered_file;
-    if ($params->{'overwrite'}){
-	$filtered_file = $params->{'path'}.'/'.$params->{'file'};
-    }else{
-	$filtered_file = $Conf::Conf{'tmpdir'}.'/'.$params->{'file'};
-    }
-    my $file_to_archive = $params->{'path'}.'/'.$params->{'file'};
-    if (my $msg = new Message($file_to_archive,1)){
-	if($msg->clean_html()){
-	    if(open TMP, ">$filtered_file") {
-		print TMP $msg->{'msg'}->as_string;
-		close TMP;
-		$file_to_archive = $filtered_file;
-	    }else{
-		&do_log('err','Unable to create a tmp file to write clean HTML to file %s',$filtered_file);
-		return undef;
-	    }
-	}else{
-	    &do_log('err','HTML cleaning in file %s failed.',$file_to_archive);
-	    return undef;
-	}
-    }else{
-	&do_log('err','Unable to create a Message object with file %s',$file_to_archive);
-	return undef;
-    }
-    return $file_to_archive;
-}
-
 =pod 
 
 =head1 AUTHORS 
