@@ -2,11 +2,14 @@
 
 # arc2webarc.pl - This script will concert existing Sympa mail archives
 # to Sympa web archives
-# RCS Identication ; $Revision: 6360 $ ; $Date: 2010-02-25 08:10:00 +0100 (jeu 25 fév 2010) $ 
+# RCS Identication ; $Revision: 10083 $ ; $Date: 2014-01-01 01:12:12 +0100 (mer. 01 janv. 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +22,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 use lib '/usr/local/libdata/perl5/site_perl/Sympa';
 use wwslib;
@@ -216,7 +218,7 @@ sub store_messages {
 		$date = $1;
 		
 		# Date type : Mon, 8 Dec 97 13:33:47 +0100
-		if ($date =~ /^\w{2,3},\s+\d{1,2}\s+([\w]{2,3})\s+(\d{2,4})/) {
+		if ($date =~ /^\w{2,3},\s+\d{1,2}\s+([\w\x80-\xFF]{2,3})\s+(\d{2,4})/) {
 		    $month = $1;
 		    $year =$2;
 		    #print STDERR "$month/$year\n";
@@ -244,7 +246,8 @@ sub store_messages {
 		
 		# Month format
 		if ($month !~ /^\d+$/) {
-		    $month =~ y/A-Z/eua-z/;
+		    $month =~ y/\xe9\xfb/eu/; #FIXME
+		    $month =~ y/A-Z/a-z/;
 		    if (!$month_idx{$month}) {
 			$incorrect = 1;
 		    }else {
