@@ -110,7 +110,7 @@ foreach my $alias (@aliases) {
     next if ($alias =~ /^\#/);
     next if ($alias =~ /^\s*$/);
     
-    $alias =~ /^([^:]+):\s*(\".*\")$/;
+    $alias =~ /^([^\s:]+)[\s:]+(\".*\")$/;
     my $alias_value = $1;
     my $command_value = $2;
     
@@ -120,7 +120,7 @@ foreach my $alias (@aliases) {
 	$command_value = "sympa";
     } 
     
-    # We substitute all occurences of + by - for the rest of the attributes, including the dn.
+    # We substitute all occurrences of + by - for the rest of the attributes, including the dn.
     # The rationale behind this is that the "uid" attribute prevents the use of the '+' character.
     $alias_value =~ s/\+/\-/g;
     
@@ -179,7 +179,7 @@ elsif ($operation eq 'del') {
 	    next;
 	}
 	
-	$alias =~ /^([^:]+):/; 
+	$alias =~ /^([^\s:]+)[\s:]/; 
 	my $alias_value = $1;
 	$alias_value =~ s/\+/\-/g;
 
@@ -214,12 +214,12 @@ sub already_defined {
 
     foreach my $alias (@aliases) {
 	
-	$alias =~ /^([^:]+):/;
+	$alias =~ /^([^\s:]+)[\s:]/;
 
 	my $source_result = $ldap_connection->search(filter => "(".$ldap_mail_attribute."=".$1."\@".$domain.")",
 						     base => $ldap_base_dn);
 	if ($source_result->count != 0) {
-	    print STRERR "Alias already defined : $1\n";
+	    print STDERR "Alias already defined : $1\n";
 	    &finalize_ldap;
 	    return 1;
 	}
@@ -272,15 +272,18 @@ sub initialize_ldap {
     {
     	if ($ldap_cachain)
     	{
-    		unless ($ldap_connection = Net::LDAPS->new($ldap_host), version => 3, verify => 'require', sslversion=> 'sslv3',
-        		                  cafile => $ldap_cachain) {
+    		unless ($ldap_connection = Net::LDAPS->new(
+			$ldap_host, version => 3, verify => 'require',
+			sslversion=> 'sslv3', cafile => $ldap_cachain)) {
 			print STDERR "Can't connect to LDAP server using SSL or unable to verify Server certificate for $ldap_host: $@\n";
 			return 0;
     		}
     	}
     	else
     	{
-    		unless ($ldap_connection = Net::LDAPS->new($ldap_host), version => 3, verify => 'none', sslversion=> 'sslv3') {
+    		unless ($ldap_connection = Net::LDAPS->new(
+			$ldap_host, version => 3, verify => 'none',
+			sslversion=> 'sslv3')) {
 			print STDERR "Can't connect to LDAP server using SSL for $ldap_host: $@\n";
 			return 0;
     		}
@@ -288,7 +291,7 @@ sub initialize_ldap {
     }        
     else 
     {
-    	unless ($ldap_connection = Net::LDAP->new($ldap_host), version => 3) {
+    	unless ($ldap_connection = Net::LDAP->new($ldap_host, version => 3)) {
 		print STDERR "Can't connect to LDAP server $ldap_host: $@\n";
 		return 0;
     	}
