@@ -85,7 +85,7 @@ if ($operation eq 'add') {
 	    next;
 	}
 	
-	$alias =~ /^([^:]+):\s*(\".*\")$/;
+	$alias =~ /^([^\s:]+)[\s:]+(\".*\")$/;
 	my $alias_value = $1;
 	my $command_value = $2;
 	$command_value =~ s/\"//g;
@@ -112,7 +112,7 @@ elsif ($operation eq 'del') {
 	    next;
 	}
 	
-	$alias =~ /^([^:]+):/; 
+	$alias =~ /^([^\s:]+)[\s:]/; 
 	my $alias_value = $1;
 	$alias_value =~ s/\+/\-/g;
 
@@ -141,7 +141,7 @@ sub already_defined {
 	    next;
 	}
 	
-	$alias =~ /^([^:]+):/; 
+	$alias =~ /^([^\s:]+)[\s:]/; 
 	my $alias_value = $1;
 	$alias_value =~ s/\+/\-/g;	
     
@@ -149,7 +149,7 @@ sub already_defined {
         $dbh->do($sql) or die "$sql : $DBI::errstr";
 
         @enr = $dbh->selectrow_array($sql);
-	if (@enr[0] != 0) {
+	if ($enr[0] != 0) {
 	    print STDERR "Alias already defined : $alias_value\n";
 	    &finalize_mysql;
 	    return 1;
