@@ -2,11 +2,14 @@
 
 # bounced.pl - This script runs as a daemon ; it does the incoming 
 # non-delivery reports analysis and storage
-# RCS Identication ; $Revision: 7451 $ ; $Date: 2012-07-06 11:21:24 +0200 (ven 06 jui 2012) $ 
+# RCS Identication ; $Revision: 11009 $ ; $Date: 2014-06-18 09:54:30 +0200 (mer. 18 juin 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +22,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
 ## Worl Wide Sympa is a front-end to Sympa Mailing Lists Manager
@@ -55,7 +57,7 @@ my %equiv = ( "user unknown" => '5.1.1',
 	      "receiver not found" => '5.1.1',
 	      "the recipient name is not recognized" => '5.1.1',
 	      "sorry, no mailbox here by that name" => '5.1.1',
-	      "utilisateur non recens dans le carnet d'adresses public" => '5.1.1',
+	      "utilisateur non recens\xE9 dans le carnet d'adresses public" => '5.1.1',
 	      "unknown address" => '5.1.1',
 	      "unknown user" => '5.1.1',
 	      "550" => '5.1.1',
@@ -142,8 +144,8 @@ unless ($main::options{'debug'} || $main::options{'foreground'}) {
 my $options;
 $options->{'stderr_to_tty'} = 1 if ($main::options{'foreground'});
 
-## Create and write the pidfile
-&tools::write_pid($wwsconf->{'bounced_pidfile'}, $$, $options);
+## Create and write the PID file
+tools::write_pid('bounced', $$, $options);
 
 if ($main::options{'log_level'}) {
     &Log::set_log_level($main::options{'log_level'});
@@ -210,7 +212,9 @@ while (!$end) {
 
 	last if $end;
 	next unless (-f "$queue/$file");
-	
+
+	next if $file =~ /\.(lock|NFSLock)\z/; # Skip lock.
+
 	unless ($file =~ /^(\S+)\.\d+\.\d+$/) {
 	    my @s = stat("$queue/$file");
 	    if (POSIX::S_ISREG($s[2])) {
@@ -252,15 +256,23 @@ while (!$end) {
 	$parser->output_to_core(1);
 	my $entity = $parser->read(\*BOUNCE);
 	my $head = $entity->head;
-	my $to = $head->get('to', 0);
+
+	## Some MTAs decorate To: field of DSN as "mailbox <address>".
+	## Pick address only.
+	my $to;
+	my $to_header = $head->get('to', 0);
+	if ($to_header) {
+	    my @to = Mail::Address->parse($to_header);
+	    if (@to and $to[0] and $to[0]->address) {
+		$to = $to[0]->address;
+	    }
+	}
+
 	close BOUNCE ; 
 
 	my $who;
-	chomp $to;	
 	&do_log('debug', 'bounce for :%s:  Conf{bounce_email_prefix}=%sxx',$to,$Conf{'bounce_email_prefix'});
-	$to =~ s/<//;
-	$to =~ s/>//;
-	if ($to =~ /^$Conf{'bounce_email_prefix'}\+(.*)\@(.*)$/) { #VERP in use
+	if ($to and $to =~ /^$Conf{'bounce_email_prefix'}\+(.*)\@(.*)$/) { #VERP in use
 
 	    my $local_part = $1;
 	    my $robot = $2;
@@ -550,7 +562,7 @@ while (!$end) {
 
 }
 do_log('notice', 'bounced exited normally due to signal');
-&tools::remove_pid($wwsconf->{'bounced_pidfile'}, $$);
+tools::remove_pid('bounced', $$);
 
 exit(0);
 
