@@ -1,9 +1,12 @@
 # Language.pm - This module does just the initial setup for the international messages
-# RCS Identication ; $Revision: 7533 $ ; $Date: 2012-07-27 15:13:43 +0200 (ven 27 jui 2012) $ 
+# RCS Identication ; $Revision: 11132 $ ; $Date: 2014-07-08 05:23:07 +0200 (mar. 08 juil. 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +19,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Language;
 
@@ -103,6 +105,7 @@ my %template2textdomain = ('help_admin.tt2' => 'web_help',
 			   'help_mail_commands.tt2' => 'web_help',
 			   'help_sendmsg.tt2' => 'web_help',
 			   'help_shared.tt2' => 'web_help',
+			   'help_suspend.tt2' => 'web_help',
 			   'help.tt2' => 'web_help',
 			   'help_user_options.tt2' => 'web_help',
 			   'help_user.tt2' => 'web_help',
@@ -224,6 +227,23 @@ sub Lang2Locale {
     return $lang2locale{$lang} || $lang;
 }
 
+# Convert language code or locale name to IETF language tag (RFC 5646).
+# Note: Format of result is "xx" or "xx-XX".  Vague region subtag will be
+# omitted.
+sub LanguageTag {
+    my $lang = shift;
+    my @parts = split /[\W_]/, $lang;
+
+    if ({reverse %lang2locale}->{$lang} and
+	{reverse %lang2locale}->{$lang} eq $parts[0]) {
+	return $parts[0];
+    } elsif (scalar @parts > 1 and length $parts[1]) {
+	return join '-', $parts[0], $parts[1];
+    } else {
+	return $parts[0];
+    }
+}
+
 sub maketext {
     my $template_file = shift;
     my $msg = shift;
@@ -283,7 +303,7 @@ sub sympa_dgettext {
 	return '';
     }
 
-    return &Locale::Messages::dgettext($textdomain, @param);
+    return Locale::Messages::dgettext($textdomain, $param[0]);
 
 }
 
@@ -320,7 +340,7 @@ sub gettext {
 	return '';
     }
 
-    return &Locale::Messages::gettext(@param);
+    return Locale::Messages::gettext($param[0]);
 
 }
 
