@@ -2,8 +2,11 @@
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,16 +19,18 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Scenario;
 
 use strict;
 
+use Mail::Address;
 use Net::Netmask;
+# tentative
+use Data::Dumper;
 
-use tools;
+#use tools; # temporarily disabled to avoid dependency loop.
 use List;
 use Log;
 use Conf;
@@ -142,7 +147,7 @@ sub new {
 
     ## Keep the scenario in memory
     $all_scenarios{$scenario->{'file_path'}} = $scenario;
-    
+
     return $scenario;
 }
 
@@ -156,7 +161,8 @@ sub _parse_scenario {
     my @scenario;
     my @rules = split /\n/, $paragraph;
 
-    foreach my $current_rule (@rules) {
+    foreach my $current_rule (@rules) {	
+	my @auth_methods_list;
 	next if ($current_rule =~ /^\s*\w+\s*$/o); # skip paragraph name
 	my $rule = {};
 	$current_rule =~ s/\#.*$//;         # remove comments
@@ -171,41 +177,29 @@ sub _parse_scenario {
         
 	if ($current_rule =~ /\s*(include\s*\(?\'?(.*)\'?\)?)\s*$/i) {
 	    $rule->{'condition'} = $1;
-	}elsif ($current_rule =~ /^\s*(.*)\s+(md5|pgp|smtp|smime|dkim)((\s*,\s*(md5|pgp|smtp|smime|dkim))*)\s*->\s*(.*)\s*$/i) {
-	    $rule->{'condition'}=$1;
-	    $rule->{'auth_method'}=$2 || 'smtp';
-	    $rule->{'action'} = $6;
+	    push(@scenario, $rule);
+	}elsif ($current_rule =~ /^\s*(.*?)\s+((\s*(md5|pgp|smtp|smime|dkim)\s*,?)*)\s*->\s*(.*)\s*$/gi) {
+	    $rule->{'condition'} = $1;
+	    $rule->{'action'} = $5;
+	    my $auth_methods = $2 || 'smtp';
+	    $auth_methods=~ s/\s//g;
+	    @auth_methods_list = split ',', $auth_methods;
 	}else {
 	    do_log('err', "error rule syntaxe in scenario $function rule line $. expected : <condition> <auth_mod> -> <action>");
 	    do_log('err',"error parsing $current_rule");
 	    return undef;
 	}
 
-	
-#	## Make action an ARRAY
-#	my $action = $6;
-#	my @actions;
-#	while ($action =~ s/^\s*((\w+)(\s?\([^\)]*\))?)(\s|\,|$)//) {
-#	    push @actions, $1;
-#	}
-#	$rule->{action} = \@actions;
-	       
-	push(@scenario,$rule);
-
 	## Duplicate the rule for each mentionned authentication method
-        my $auth_list = $3 ; 
-        while ($auth_list =~ /\s*,\s*(md5|pgp|smtp|smime|dkim)((\s*,\s*(md5|pgp|smtp|smime|dkim))*)\s*/i) {
+	foreach my $auth_method (@auth_methods_list) {
 	    push(@scenario,{'condition' => $rule->{condition}, 
-                            'auth_method' => $1,
+                            'auth_method' => $auth_method,
                             'action' => $rule->{action}});
-	    $auth_list = $2;
-#	    do_log('debug3', "load rule ite: $rule->{'condition'} $1 -> $rule->{'action'}");
 	}
-	
     }
-    
+
     $structure->{'rules'} = \@scenario;
-   
+
     return $structure; 
 }
 
@@ -400,7 +394,6 @@ sub request_action {
 	## Add rules at the beginning of the array
 	unshift @rules, @{$include_scenario->{'rules'}};
     }
-
     ## Look for 'include' directives amongst rules first
     foreach my $index (0..$#rules) {
 	if ($rules[$index]{'condition'} =~ /^\s*include\s*\(?\'?([\w\.]+)\'?\)?\s*$/i) {
@@ -423,7 +416,7 @@ sub request_action {
 	# &do_log('info', 'List::request_action : verify rule %s',$rule->{'condition'});
 
 	if ($auth_method eq $rule->{'auth_method'}) {
-	    my $result =  &verify ($context,$rule->{'condition'});
+		my $result = verify( $context, $rule->{'condition'}, $rule );
 
 	    ## Cope with errors
 	    if (! defined ($result)) {
@@ -525,15 +518,11 @@ sub request_action {
 
 ## check if email respect some condition
 sub verify {
-    my ($context, $condition) = @_;
+	my ($context, $condition, $rule ) = @_;
     &do_log('debug2', '(%s)', $condition);
     
     my $robot = $context->{'robot_domain'};
     
-#    while (my($k,$v) = each %{$context}) {
-#	do_log('debug3',"verify: context->{$k} = $v");
-#    }
-    
     unless (defined($context->{'sender'} )) {
 	do_log('info',"internal error, no sender find in List::verify, report authors");
 	return undef;
@@ -558,8 +547,8 @@ sub verify {
     
     if (defined ($context->{'msg'})) {
 	my $header = $context->{'msg'}->head;
-	unless (($header->get('to') && ($header->get('to') =~ /$context->{'listname'}/i)) || 
-		($header->get('cc') && ($header->get('cc') =~ /$context->{'listname'}/i))) {
+	unless (($header->get('to') && (join(', ', $header->get('to')) =~ /$context->{'listname'}/i)) || 
+		($header->get('cc') && (join(', ', $header->get('cc')) =~ /$context->{'listname'}/i))) {
 	    $context->{'is_bcc'} = 1;
 	}else{
 	    $context->{'is_bcc'} = 0;
@@ -581,8 +570,9 @@ sub verify {
     
     ## The expression for regexp is tricky because we don't allow the '/' character (that indicates the end of the regexp
     ## but we allow any number of \/ escape sequence)
-    while ($arguments =~ s/^\s*(
-				\[\w+(\-\>[\w\-]+)?\]
+    while (
+	$arguments =~ s/^\s*(
+				(\[\w+(\-\>[\w\-]+)?\](\[[-+]?\d+\])?)
 				|
 				([\w\-\.]+)
 				|
@@ -601,6 +591,11 @@ sub verify {
 	    $value =~ s/\[custom_vars\-\>([\w\-]+)\]/$context->{'custom_vars'}{$1}/;
 	}
 	
+	## Family vars
+	if ($value =~ /\[family\-\>([\w\-]+)\]/i) {
+	    $value =~ s/\[family\-\>([\w\-]+)\]/$context->{'family'}{$1}/;
+	}
+	
 	## Config param
 	elsif ($value =~ /\[conf\-\>([\w\-]+)\]/i) {
 	    if (my $conf_value = &Conf::get_robot_conf($robot, $1)) {
@@ -611,9 +606,9 @@ sub verify {
 		# a condition related to a undefined context variable is always false
 		return -1 * $negation;
 	    }
-	    
-	    ## List param
-	}elsif ($value =~ /\[list\-\>([\w\-]+)\]/i) {
+	}
+	## List param
+	elsif ($value =~ /\[list\-\>([\w\-]+)\]/i) {
 	    my $param = $1;
 
 	    if ($param =~ /^(name|total)$/) {
@@ -621,8 +616,8 @@ sub verify {
 	    }elsif ($param eq 'address') {
 		my $list_address = $list->get_list_address();
 		$value =~ s/\[list\-\>([\w\-]+)\]/$list_address/;
-	    
-	    }elsif ($list->{'admin'}{$param} and (!ref($list->{'admin'}{$param})) ) {
+	    } elsif (exists $::pinfo{$param}
+		and !ref($list->{'admin'}{$param})) {
 		$value =~ s/\[list\-\>([\w\-]+)\]/$list->{'admin'}{$param}/;
 	    }else{
 		do_log('err','Unknown list parameter %s in rule %s', $value, $condition);
@@ -649,21 +644,36 @@ sub verify {
 	    $context->{'subscriber'} ||= $list->get_subscriber($context->{'sender'});
 	    $value =~ s/\[subscriber\-\>([\w\-]+)\]/$context->{'subscriber'}{$1}/;
 	    
-	    ## SMTP Header field
-	}elsif ($value =~ /\[(msg_header|header)\-\>([\w\-]+)\]/i) {
+	} elsif ($value =~
+	    /\[(msg_header|header)\-\>([\w\-]+)\](?:\[([-+]?\d+)\])?/i) {
+	    ## SMTP header field.
+	    ## "[msg_header->field] returns arrayref of field values,
+	    ## preserving order. "[msg_header->field][index]" returns one
+	    ## field value.
 	    my $field_name = $2;
+	    my $index = (defined $3) ? $3 + 0 : undef;
 	    if (defined ($context->{'msg'})) {
-		my $header = $context->{'msg'}->head;
-		my @fields = $header->get($field_name);
-		## Defaulting empty or missing fields to '', so that we can test
-		## their value in Scenario, considering that, for an incoming message,
-		## a missing field is equivalent to an empty field : the information it
-		## is supposed to contain isn't available.
-		unless (@fields) {
-		    @fields = ('');
+		my $headers = $context->{'msg'}->head->header();
+		my @fields = grep { $_ } map {
+		    my ($h, $v) = split /\s*:\s*/, $_, 2;
+		    (lc $h eq lc $field_name) ? $v : undef;
+		} @{$headers || []};
+		## Defaulting empty or missing fields to '', so that we can
+		## test their value in Scenario, considering that, for an
+		## incoming message, a missing field is equivalent to an empty
+		## field : the information it is supposed to contain isn't
+		## available.
+		if (defined $index) {
+		    $value = $fields[$index];
+		    unless (defined $value) {
+			$value = '';
+		    }
+		} else {
+		    unless (@fields) {
+		        @fields = ('');
+		    }
+		    $value = \@fields;
 		}
-		
-		$value = \@fields;
 	    }else {
 		return -1 * $negation;
 	    }
@@ -679,14 +689,12 @@ sub verify {
 	    return -1 * $negation unless (defined ($context->{'msg'}));
 	    
 	    my @bodies;
-	    my @parts = $context->{'msg'}->parts();
-	    
-	    ## Should be recurcive...
-	    foreach my $i (0..$#parts) {
-		next unless ($parts[$i]->effective_type() =~ /^text/);
-		next unless (defined $parts[$i]->bodyhandle);
+	    ## FIXME:Should be recurcive...
+	    foreach my $part ($context->{'msg'}->parts) {
+		next unless ($part->effective_type() =~ /^text/);
+		next unless (defined $part->bodyhandle);
 		
-		push @bodies, $parts[$i]->bodyhandle->as_string();
+		push @bodies, $part->bodyhandle->as_string();
 	    }
 	    $value = \@bodies;
 	    
@@ -694,12 +702,17 @@ sub verify {
 	    return -1 * $negation unless (defined ($context->{'msg'}));
 	    
 	    my @types;
-	    my @parts = $context->{'msg'}->parts();
-	    foreach my $i (0..$#parts) {
-		push @types, $parts[$i]->effective_type();
+	    foreach my $part ($context->{'msg'}->parts) {
+		push @types, $part->effective_type();
 	    }
 	    $value = \@types;
 
+	}elsif ($value =~ /\[msg\-\>(\w+)\]/i) {
+	    return -1 * $negation unless (defined ($context->{'message'}));
+	    my $message_field = $1;
+	    return -1 * $negation unless (defined ($context->{'message'}{$message_field}));
+	    $value = $context->{'message'}{$message_field};
+
 	}elsif ($value =~ /\[current_date\]/i) {
 	    my $time = time;
 	    $value =~ s/\[current_date\]/$time/;
@@ -761,12 +774,26 @@ sub verify {
     }
     ##### condition is_listmaster
     if ($condition_key eq 'is_listmaster') {
-	
-	if ($args[0] eq 'nobody') {
+	if (!ref $args[0] and $args[0] eq 'nobody') {
 	    return -1 * $negation ;
 	}
 
-	if ( &List::is_listmaster($args[0],$robot)) {
+	my @arg;
+	my $ok = undef;
+	if (ref $args[0] eq 'ARRAY') {
+	    @arg = map { $_->address }
+	    grep { $_ } map { (Mail::Address->parse($_)) } @{$args[0]};
+	} else {
+	    @arg = map { $_->address }
+	    grep { $_ } Mail::Address->parse($args[0]);
+	}
+	foreach my $arg (@arg) {
+	    if (List::is_listmaster($arg, $robot)) {
+		$ok = $arg;
+		last;
+	    }
+	}
+	if ($ok) {
 	    return $negation;
 	}else{
 	    return -1 * $negation;
@@ -829,23 +856,50 @@ sub verify {
 	    return -1 * $negation ;
 	}
 
-	if ($condition_key eq 'is_subscriber') {
+	my @arg;
+	my $ok = undef;
+	if (ref $args[1] eq 'ARRAY') {
+	    @arg = map { $_->address }
+	    grep { $_ } map { (Mail::Address->parse($_)) } @{$args[1]};
+	} else {
+	    @arg = map { $_->address }
+	    grep { $_ } Mail::Address->parse($args[1]);
+	}
 
-	    if ($list2->is_user($args[1])) {
+	if ($condition_key eq 'is_subscriber') {
+	    foreach my $arg (@arg) {
+		if ($list2->is_user($arg)) {
+		    $ok = $arg;
+		    last;
+		}
+	    }
+	    if ($ok) {
 		return $negation ;
 	    }else{
 		return -1 * $negation ;
 	    }
 
 	}elsif ($condition_key eq 'is_owner') {
-	    if ($list2->am_i('owner',$args[1])) {
+	    foreach my $arg (@arg) {
+		if ($list2->am_i('owner', $arg)) {
+		    $ok = $arg;
+		    last;
+		}
+	    }
+	    if ($ok) {
 		return $negation ;
 	    }else{
 		return -1 * $negation ;
 	    }
 
 	}elsif ($condition_key eq 'is_editor') {
-	    if ($list2->am_i('editor',$args[1])) {
+	    foreach my $arg (@arg) {
+		if ($list2->am_i('editor', $arg)) {
+		    $ok = $arg;
+		    last;
+		}
+	    }
+	    if ($ok) {
 		return $negation ;
 	    }else{
 		return -1 * $negation ;
@@ -869,16 +923,29 @@ sub verify {
             $regexp =~ s/\[host\]/$reghost/g ;
 	}
 
+	# wrap matches with eval{} to avoid crash by malformed regexp.
+	my $r = 0;
 	if (ref($args[0])) {
-	    foreach my $arg (@{$args[0]}) {
-		return $negation 
-		    if ($arg =~ /$regexp/i);
-	    }
-	}else {
-	    if ($args[0] =~ /$regexp/i) {
-		return $negation ;
-	    }
+	    eval {
+		foreach my $arg (@{$args[0]}) {
+		    if ($arg =~ /$regexp/i) {
+			$r = 1;
+			last;
+		    }
+		}
+	    };
+	} else {
+	    eval {
+		if ($args[0] =~ /$regexp/i) {
+		    $r = 1;
+		}
+	    };
 	}
+	if ($@) {
+	    &do_log('err', 'cannot evaluate match: %s', $@);
+	    return undef;
+	}
+	return $negation if $r;
 	
 	return -1 * $negation ;
 
@@ -907,12 +974,11 @@ sub verify {
 	    foreach my $arg (@{$args[0]}) {
 		&do_log('debug3', 'ARG: %s', $arg);
 		return $negation 
-		    if ($arg =~ /^$args[1]$/i);
-	    }
-	}else {
-	    if ($args[0] =~ /^$args[1]$/i) {
-		return $negation ;
+		    if lc($arg) eq lc($args[1]);
 	    }
+	} else {
+	    return $negation
+		if lc($args[0]) eq lc($args[1]);
 	}
 	return -1 * $negation ;
     }
@@ -921,7 +987,7 @@ sub verify {
     if ($condition_key =~ /^customcondition::(\w+)/o ) {
     	my $condition = $1;
     	
-    	my $res = &verify_custom($condition, \@args, $robot, $list);
+    	my $res = &verify_custom($condition, \@args, $robot, $list, $rule );
 	return undef unless defined $res;
 	return $res * $negation ;
     }
@@ -1030,9 +1096,10 @@ sub search{
  
         my $res = $ds->fetch;
         $ds->disconnect();
-        do_log('debug2','Result of SQL query : %d = %s', $res->[0], $statement);
+        my $first_row = ref($res->[0]) ? $res->[0]->[0] : $res->[0];
+        do_log('debug2','Result of SQL query : %d = %s', $first_row, $statement);
  
-        if ($res->[0] == 0){
+        if ($first_row == 0){
             $persistent_cache{'named_filter'}{$filter_file}{$filter}{'value'} = 0;
         }else {
             $persistent_cache{'named_filter'}{$filter_file}{$filter}{'value'} = 1;
@@ -1149,9 +1216,9 @@ sub search{
 	    while (<FILE>) {
 		# &do_log('debug3', 'List::search: eval rule %s', $_);
 		next if (/^\s*$/o || /^[\#\;]/o);
-		my $regexp= $_ ;
-		chomp $regexp;
-		$regexp =~ s/\*/.*/ ; 
+                chomp;
+		my $regexp= "\Q$_\E" ;
+		$regexp =~ s/\\\*/.*/ ; 
 		$regexp = '^'.$regexp.'$';
 		# &do_log('debug3', 'List::search: eval  %s =~ /%s/i', $sender,$regexp);
 		return 1  if ($sender =~ /$regexp/i);
@@ -1166,7 +1233,7 @@ sub search{
 
 # eval a custom perl module to verify a scenario condition
 sub verify_custom {
-	my ($condition, $args_ref, $robot, $list) = @_;
+	my ($condition, $args_ref, $robot, $list, $rule ) = @_;
         my $timeout = 3600;
 	
 	my $filter = join ('*', @{$args_ref});
@@ -1190,8 +1257,10 @@ sub verify_custom {
 	    &do_log('err', 'Error requiring %s : %s (%s)', $condition, "$@", ref($@));
 	    return undef;
 	}
-	my $res;
-	eval "\$res = CustomCondition::${condition}::verify(\@{\$args_ref});";
+	my $res = do {
+		local $_ = $rule;
+		eval "CustomCondition::${condition}::verify(\@{\$args_ref})"
+	};
 	if ($@) {
 	    &do_log('err', 'Error evaluating %s : %s (%s)', $condition, "$@", ref($@));
 	    return undef;
@@ -1210,4 +1279,16 @@ sub dump_all_scenarios {
     close TMP;
 }
 
+sub is_purely_closed {
+    my $self = shift;
+    foreach my $rule (@{$self->{'rules'}}) {
+	if ($rule->{'condition'} ne 'true' && $rule->{'action'} !~ /reject/) {
+	    Log::do_log('debug2','Scenario %s is not purely closed.',$self->{'title'});
+	    return 0;
+	}
+    }
+    Log::do_log('notice','Scenario %s is purely closed.',$self->{'file_path'});
+    return 1;
+}
+
 1;
