@@ -1,10 +1,13 @@
 # Upgrade.pm - This module gathers all subroutines used to upgrade Sympa data structures
-#<!-- RCS Identication ; $Revision: 7449 $ --> 
+#<!-- RCS Identication ; $Revision: 10441 $ --> 
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +20,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Upgrade;
 
@@ -811,8 +813,10 @@ sub probe_db {
 						      'email_netidmap' => 'varchar(100)',
 						      'robot_netidmap' => 'varchar(80)'},
 				 'session_table' => {'id_session' => 'varchar(30)',
+						     'prev_id_session' => 'varchar(30)',
 						     'start_date_session' => 'int(11)',
 						     'date_session' => 'int(11)',
+						     'refresh_date_session' => 'int(11)',
 						     'remote_addr_session' => 'varchar(60)',
 						     'robot_session'  => 'varchar(80)',
 						     'email_session'  => 'varchar(100)',
@@ -856,7 +860,7 @@ sub probe_db {
 						       'messageid_bulkspool' => 'varchar(200)',
 						       'message_bulkspool' => 'longtext',
 						       'lock_bulkspool' => 'int(1)',
-						       'dkim_privatekey_bulkspool' => 'varchar(1000)',
+						       'dkim_privatekey_bulkspool' => 'varchar(2000)',
 						       'dkim_selector_bulkspool' => 'varchar(50)',
 						       'dkim_d_bulkspool' => 'varchar(50)',
 						       'dkim_i_bulkspool' => 'varchar(100)',
@@ -929,8 +933,10 @@ sub probe_db {
 						       'email_netidmap' => 'text',
 						       'robot_netidmap' => 'text'},
 				  'session_table' => {'id_session' => 'text',
+						     'prev_id_session' => 'text',
 						     'start_date_session' => 'integer',
 						     'date_session' => 'integer',
+						     'refresh_date_session' => 'integer',
 						     'remote_addr_session' => 'text',
 						     'robot_session'  => 'text',
 						     'email_session'  => 'text',
@@ -1149,7 +1155,27 @@ sub probe_db {
 	    $t =~ s/^"main"\.//; # needed for SQLite 3
 	    $t =~ s/^.*\"([^"]+)\"$/$1/;
  	}
-	
+
+    ## Check required tables
+    foreach my $t1 (keys %{$db_struct{'SQLite'}}) {
+        my $found;
+        foreach my $t2 (@tables) {
+        $found = 1 if ($t1 eq $t2);
+        }
+        unless ($found) {
+        unless ($dbh->do("CREATE TABLE $t1 (temporary integer)")) {
+            &do_log('err', 'Could not create table %s in database %s : %s', $t1, $Conf::Conf{'db_name'}, $dbh->errstr);
+            next;
+        }
+        
+        push @report, sprintf('Table %s created in database %s', $t1, $Conf::Conf{'db_name'});
+        &do_log('notice', 'Table %s created in database %s', $t1, $Conf::Conf{'db_name'});
+        push @tables, $t1;
+        $real_struct{$t1} = {};
+        }
+    }
+
+    ## Get fields
 	foreach my $t (@tables) {
 	    next unless (defined $db_struct{$Conf::Conf{'db_type'}}{$t});
 	    
@@ -1175,8 +1201,8 @@ sub probe_db {
 	    }
 	}
 	
-	# Une simple requte sqlite : PRAGMA table_info('nomtable') , retourne la liste des champs de la table en question.
-	# La liste retourne est compose d'un NOrdre, Nom du champ, Type (longueur), Null ou not null (99 ou 0),Valeur par dfaut,Cl primaire (1 ou 0)
+	# Une simple requête sqlite : PRAGMA table_info('nomtable') , retourne la liste des champs de la table en question.
+	# La liste retournée est composée d'un N°Ordre, Nom du champ, Type (longueur), Null ou not null (99 ou 0),Valeur par défaut,Clé primaire (1 ou 0)
 	
     }elsif ($Conf::Conf{'db_type'} eq 'Oracle') {
  	
@@ -1230,7 +1256,7 @@ sub probe_db {
     
     my $found_tables = 0;
     foreach $table('user_table', 'subscriber_table', 'admin_table') {
-	if ($checked{$table} || $checked{'public.' . $table}) {
+	if ($checked{$table} || $checked{'public.' . $table} || $checked{$Conf::Conf{'db_user'} . '.' . $table}) {
 	    $found_tables++;
 	}else {
 	    &do_log('err', 'Table %s not found in database %s', $table, $Conf::Conf{'db_name'});
@@ -1272,8 +1298,15 @@ sub probe_db {
 		    
 		    ## Remove temporary DB field
 		    if ($real_struct{$t}{'temporary'}) {
-			unless ($dbh->do("ALTER TABLE $t DROP temporary")) {
-			    &do_log('err', 'Could not drop temporary table field : %s', $dbh->errstr);
+			if ($Conf::Conf{'db_type'} eq 'SQLite') {
+			    # FIXME:
+			    # SQLite doesn't support "ALTER TABLE DROP COLUMN".
+			} else {
+			    unless ($dbh->do(
+				"ALTER TABLE $t DROP COLUMN temporary"
+			    )) {
+				&do_log('err', 'Could not drop temporary table field : %s', $dbh->errstr);
+			    }
 			}
 			delete $real_struct{$t}{'temporary'};
 		    }
@@ -1282,35 +1315,87 @@ sub probe_db {
 		}
 		
 		## Change DB types if different and if update_db_types enabled
-		if ($Conf::Conf{'update_db_field_types'} eq 'auto' && $Conf::Conf{'db_type'} ne 'SQLite') {
+		if ($Conf::Conf{'update_db_field_types'} eq 'auto') {
 		    unless (&check_db_field_type(effective_format => $real_struct{$t}{$f},
-						 required_format => $db_struct{$Conf::Conf{'db_type'}}{$t}{$f})) {
+			required_format => $db_struct{$Conf::Conf{'db_type'}}{$t}{$f})) {
 			push @report, sprintf('Field \'%s\'  (table \'%s\' ; database \'%s\') does NOT have awaited type (%s). Attempting to change it...', 
-					      $f, $t, $Conf::Conf{'db_name'}, $db_struct{$Conf::Conf{'db_type'}}{$t}{$f});
+			    $f, $t, $Conf::Conf{'db_name'}, $db_struct{$Conf::Conf{'db_type'}}{$t}{$f});
 			&do_log('notice', 'Field \'%s\'  (table \'%s\' ; database \'%s\') does NOT have awaited type (%s). Attempting to change it...', 
-				$f, $t, $Conf::Conf{'db_name'}, $db_struct{$Conf::Conf{'db_type'}}{$t}{$f});
-			
+			    $f, $t, $Conf::Conf{'db_name'}, $db_struct{$Conf::Conf{'db_type'}}{$t}{$f});
+				
 			my $options;
 			if ($not_null{$f}) {
 			    $options .= 'NOT NULL';
 			}
-			
-			push @report, sprintf("ALTER TABLE $t CHANGE $f $f $db_struct{$Conf::Conf{'db_type'}}{$t}{$f} $options");
-			&do_log('notice', "ALTER TABLE $t CHANGE $f $f $db_struct{$Conf::Conf{'db_type'}}{$t}{$f} $options");
-			unless ($dbh->do("ALTER TABLE $t CHANGE $f $f $db_struct{$Conf::Conf{'db_type'}}{$t}{$f} $options")) {
-			    &do_log('err', 'Could not change field \'%s\' in table\'%s\'.', $f, $t);
-			    &do_log('err', 'Sympa\'s database structure may have change since last update ; please check RELEASE_NOTES');
-			    return undef;
+				
+			if ($Conf::Conf{'db_type'} ne 'SQLite') {
+			    push @report, sprintf("ALTER TABLE $t CHANGE $f $f $db_struct{$Conf::Conf{'db_type'}}{$t}{$f} $options");
+			    &do_log('notice', "ALTER TABLE $t CHANGE $f $f $db_struct{$Conf::Conf{'db_type'}}{$t}{$f} $options");
+			    unless ($dbh->do("ALTER TABLE $t CHANGE $f $f $db_struct{$Conf::Conf{'db_type'}}{$t}{$f} $options")) {
+				&do_log('err', 'Could not change field \'%s\' in table\'%s\'.', $f, $t);
+				&do_log('err', 'Sympa\'s database structure may have change since last update ; please check RELEASE_NOTES');
+				return undef;
+			    }
+					
+			} else { # SQLite
+			    my @oldfields = ();
+			    foreach my $oldfield (keys %{$real_struct{$t}}) {
+				if (defined $db_struct{$Conf::Conf{'db_type'}}{${t}}{$oldfield}) {
+				    push(@oldfields, $oldfield);
+				}
+			    }
+			    my $sqloldfields = join(", ", @oldfields);
+			    my $sqlnewfieldstype = join(", ", map { "$_ $db_struct{$Conf::Conf{'db_type'}}{$t}{$_}" } keys %{$db_struct{$Conf::Conf{'db_type'}}{$t}});
+
+			    # SQLite doesn't support changes on columns types hence we use a dirty workaround here:
+			    # we create a temporary table (a clone of the table containing the field to update)
+			    # we dump all the data into it
+			    # we create a new table with the right schema
+			    # we copy all the data from the temporary table into the new one
+			    # we delete the temporay table
+			    # see: http://www.sqlite.org/faq.html#q11
+			    &do_log('notice', 'Using a dirty trick to change SQLite data type see http://www.sqlite.org/faq.html#q11');
+			    my $sqlcode = "DROP TABLE IF EXISTS ${t}_backup;";
+			    push @report, $sqlcode;
+			    &do_log('notice', $sqlcode);
+			    $dbh->do($sqlcode);
+			    $sqlcode = "CREATE TEMPORARY TABLE ${t}_backup(${sqloldfields});";
+			    push @report, $sqlcode;
+			    &do_log('notice', $sqlcode);
+			    $dbh->do($sqlcode);
+			    $sqlcode = "INSERT INTO ${t}_backup SELECT ${sqloldfields} FROM ${t};";
+			    push @report, $sqlcode;
+			    &do_log('notice', $sqlcode);
+			    $dbh->do($sqlcode);
+			    $sqlcode = "DROP TABLE ${t};";
+			    push @report, $sqlcode;
+			    &do_log('notice', $sqlcode);
+			    $dbh->do($sqlcode);
+			    $sqlcode = "CREATE TABLE ${t}(${sqlnewfieldstype});";
+			    push @report, $sqlcode;
+			    &do_log('notice', $sqlcode);
+			    $dbh->do($sqlcode);
+			    $sqlcode = "INSERT INTO ${t} (${sqloldfields}) SELECT ${sqloldfields} FROM ${t}_backup;";
+			    push @report, $sqlcode;
+			    &do_log('notice', $sqlcode);
+			    my $toto = $dbh->do($sqlcode);
+			    $sqlcode = "DROP TABLE ${t}_backup;";
+			    push @report, $sqlcode;
+			    &do_log('notice', $sqlcode);
+			    $dbh->do($sqlcode);
+			    # Update real structure
+			    foreach my $newfield (sort keys %{$db_struct{$Conf::Conf{'db_type'}}{$t}}) {
+				$real_struct{$t}{$newfield} = $db_struct{$Conf::Conf{'db_type'}}{$t}{$newfield};
+			    }
 			}
-			
 			push @report, sprintf('Field %s in table %s, structure updated', $f, $t);
 			&do_log('info', 'Field %s in table %s, structure updated', $f, $t);
 		    }
-		}else {
-		    unless ($real_struct{$t}{$f} eq $db_struct{$Conf::Conf{'db_type'}}{$t}{$f}) {
+                } else {
+                    unless ($real_struct{$t}{$f} eq $db_struct{$Conf::Conf{'db_type'}}{$t}{$f}) {
 			&do_log('err', 'Field \'%s\'  (table \'%s\' ; database \'%s\') does NOT have awaited type (%s).', $f, $t, $Conf::Conf{'db_name'}, $db_struct{$Conf::Conf{'db_type'}}{$t}{$f});
 			&do_log('err', 'Sympa\'s database structure may have change since last update ; please check RELEASE_NOTES');
-			return undef;
+                        return undef;
 		    }
 		}
 	    }
@@ -1452,7 +1537,7 @@ sub probe_db {
 		    ## drop previous index
 		    my $success;
 		    foreach my $field (@{$primary{$t}}) {
-			unless ($dbh->do("DROP INDEX $field")) {
+			unless ($dbh->do("DROP INDEX IF EXISTS $field")) {
 			    next;
 			}
 			$success = 1; last;
@@ -1466,7 +1551,7 @@ sub probe_db {
 		    }
 		    
 		    ## Add INDEX
-		    unless ($dbh->do("CREATE INDEX IF NOT EXIST $t\_index ON $t ($fields)")) {
+		    unless ($dbh->do("CREATE INDEX IF NOT EXISTS $t\_index ON $t ($fields)")) {
 			&do_log('err', 'Could not set INDEX on field \'%s\', table\'%s\'.', $fields, $t);
 			return undef;
 		    }
@@ -1606,8 +1691,9 @@ sub to_utf8 {
 
 	## If filesystem_encoding is set, files are supposed to be encoded according to it
 	my $charset;
-	if ((defined $Conf::Conf::Ignored_Conf{'filesystem_encoding'})&&($Conf::Conf::Ignored_Conf{'filesystem_encoding'} ne 'utf-8')) {
-	    $charset = $Conf::Conf::Ignored_Conf{'filesystem_encoding'};
+	if (defined $Conf::Ignored_Conf{'filesystem_encoding'} and
+	    $Conf::Ignored_Conf{'filesystem_encoding'} ne 'utf-8') {
+	    $charset = $Conf::Ignored_Conf{'filesystem_encoding'};
 	}else {	    
 	    &Language::PushLang($lang);
 	    $charset = &Language::GetCharset;
