@@ -1,11 +1,14 @@
 #! /usr/bin/perl
 # sympa.pl - This script is the main one ; it runs as a daemon and does
 # the messages/commands processing
-# RCS Identication ; $Revision: 7488 $ ; $Date: 2012-07-19 07:07:50 +0200 (jeu 19 jui 2012) $ 
+# RCS Identication ; $Revision: 11690 $ ; $Date: 2014-11-14 16:38:25 +0100 (ven. 14 nov. 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +21,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 use strict;
 
@@ -82,7 +84,8 @@ Options:
    -m, --mail                            : log calls to sendmail
    --service=process_command|process_message|process_creation  : process dedicated to messages distribution, commands or to automatic lists creation (default three of them)
    --dump=list\@dom|ALL                  : dumps subscribers 
-   --make_alias_file                     : create file in /tmp with all aliases (usefull when aliases.tpl is changed)
+   --make_alias_file [--robot=robot_name,...]
+                                         : create file in /tmp with all aliases (useful when list_aliases.tt2 is changed)
    --lowercase                           : lowercase email addresses in database
    --sync_list_db [--list=listname]      : syncs filesystem list configs to the database cache of list configs, optionally sync an individual list if specified.
    --md5_encode_password                 : rewrite password in database using md5 fingerprint. YOU CAN'T UNDO unless you save this table first
@@ -337,15 +340,16 @@ if ($signal ne 'hup') {
     do_log('debug', "Running server $$ for $service purpose ");
     unless ($main::options{'batch'} ) {
 
-	## Create and write the pidfile
-	my $file = $Conf{'pidfile'};
-	$file = $Conf{'pidfile_distribute'} if ($main::daemon_usage == DAEMON_MESSAGE) ;
-	$file = $Conf{'pidfile_creation'} if ($main::daemon_usage == DAEMON_CREATION) ;
+	## Create and write the PID file
+	my $file = 'sympa';
+	$file = 'sympa-distribute' if $main::daemon_usage == DAEMON_MESSAGE;
+	$file = 'sympa-creation' if $main::daemon_usage == DAEMON_CREATION;
 
-	## If process is running in foreground, don't write STDERR to a dedicated file
+	## If process is running in foreground, don't write STDERR to a
+	## dedicated file
 	my $options;
 	$options->{'stderr_to_tty'} = 1 if ($main::options{'foreground'});
-	&tools::write_pid($file, $$, $options);
+	tools::write_pid($file, $$, $options);
     }	
 
 
@@ -419,19 +423,78 @@ if ($main::options{'dump'}) {
     print $usage_string;
     exit 0;
 }elsif ($main::options{'make_alias_file'}) {
-    my $all_lists = &List::get_lists('*');
-    unless (open TMP, ">$Conf{'tmpdir'}/sympa_aliases.$$") {
-	printf STDERR "Unable to create $Conf{'tmpdir'}/sympa_aliases.$$, exiting\n";
-	exit;
-    }
-    printf TMP "#\n#\tAliases for all Sympa lists open (but not for robots)\n#\n";
-    close TMP;
-    foreach my $list (@$all_lists) {
-	system ("$Conf{'alias_manager'} add $list->{'name'} $list->{'domain'} $Conf{'tmpdir'}/sympa_aliases.$$") if ($list->{'admin'}{'status'} eq 'open');
-    }
-    printf ("Sympa aliases file is $Conf{'tmpdir'}/sympa_aliases.$$ file made, you probably need to installed it in your SMTP engine\n");
-    
-    exit 0;
+        my $robots = $main::options{'robot'} || '*';
+        my @robots;
+        if ($robots eq '*') {
+            @robots = List::get_robots();
+        } else {
+            @robots = grep { length $_ } split(/[\s,]+/, $robots);
+        }
+        exit 0 unless @robots;
+
+        # There may be multiple aliases files.  Give each of them suffixed
+        # name.
+        my ($basename, %robots_of, %sympa_aliases);
+        $basename = sprintf '%s/sympa_aliases.%s', $Conf::Conf{'tmpdir'}, $$;
+
+        foreach my $robot (@robots) {
+            my $file = Conf::get_robot_conf($robot, 'sendmail_aliases');
+            $robots_of{$file} ||= [];
+            push @{$robots_of{$file}}, $robot;
+        }
+        if (1 < scalar(keys %robots_of)) {
+            my $i = 0;
+            %sympa_aliases = map {
+                $i++;
+                map { $_ => sprintf('%s.%03d', $basename, $i) }
+                    @{$robots_of{$_}}
+            } sort keys %robots_of;
+        } else {
+            %sympa_aliases = map { $_ => $basename } @robots;
+        }
+
+        # Create files.
+        foreach my $sympa_aliases (values %sympa_aliases) {
+            my $fh;
+            unless (open $fh, '>', $sympa_aliases) {    # truncate if exists
+                printf STDERR "Unable to create %s: %s\n", $sympa_aliases, $!;
+                exit 1;
+            }
+            close $fh;
+        }
+
+        # Write files.
+        foreach my $robot (sort @robots) {
+            my $all_lists     = List::get_lists($robot);
+            my $alias_manager = Conf::get_robot_conf($robot, 'alias_manager');
+            my $sympa_aliases = $sympa_aliases{$robot};
+
+            my $fh;
+            unless (open $fh, '>>', $sympa_aliases) {    # append
+                printf STDERR "Unable to create %s: %s\n", $sympa_aliases, $!;
+                exit 1;
+            }
+            printf $fh "#\n#\tAliases for all Sympa lists open on %s\n#\n",
+                $robot;
+            close $fh;
+            foreach my $list (@{$all_lists || []}) {
+                next unless $list->{'admin'}{'status'} eq 'open';
+
+                system($alias_manager, 'add', $list->{'name'},
+                    $list->{'domain'}, $sympa_aliases);
+            }
+        }
+
+        if (1 < scalar(keys %robots_of)) {
+            printf
+                "Sympa aliases files %s.??? were made.  You probably need to install them in your SMTP engine.\n",
+                $basename;
+        } else {
+            printf
+                "Sympa aliases file %s was made.  You probably need to install it in your SMTP engine.\n",
+                $basename;
+        }
+        exit 0;
 }elsif ($main::options{'version'}) {
     print $version_string;
     
@@ -1036,7 +1099,9 @@ while (!$signal) {
 	my $type;
 	my $list;
 	my ($t_listname, $t_robot);
-	
+
+	next if $t_filename =~ /\.(lock|NFSLock)\z/; # Skip lock.
+
 	## test ever if it is an old bad file
 	if ($t_filename =~ /^BAD\-/i){
  	    if ((stat "$t_spool/$t_filename")[9] < (time - &Conf::get_robot_conf($robot, 'clean_delay_queue')*86400) ){
@@ -1154,10 +1219,10 @@ List::db_disconnect if ($List::dbh);
 } #end of block while ($signal ne 'term'){
 
 do_log('notice', 'Sympa exited normally due to signal');
-my $file = $Conf{'pidfile'};
-$file = $Conf{'pidfile_distribute'} if ($main::daemon_usage == DAEMON_MESSAGE) ;
-$file = $Conf{'pidfile_creation'} if ($main::daemon_usage == DAEMON_CREATION) ;
-&tools::remove_pid($file, $$);
+my $file = 'sympa';
+$file = 'sympa-distribute' if $main::daemon_usage == DAEMON_MESSAGE;
+$file = 'sympa-creation' if $main::daemon_usage == DAEMON_CREATION;
+tools::remove_pid($file, $$);
 
 exit(0);
 
@@ -1280,7 +1345,7 @@ sub DoFile {
     &report::init_report_cmd();
 	
     ## Maybe we are an automatic list
-    #_amr ici on ne doit prendre que la premire ligne !
+    #_amr ici on ne doit prendre que la première ligne !
     my ($dyn_list_family, $dyn_just_created);
     # we care of fake headers. If we put it, it's the 1st one.
     if ($hdr->as_string() =~ /^X-Sympa-Family/mo) {
@@ -1325,7 +1390,15 @@ sub DoFile {
             my $dyn_family;
             unless ($dyn_family = new Family($dyn_list_family,$robot)) {
                 &do_log('err', "Failed to process $file : family $dyn_list_family does not exist, impossible to create the dynamic list.");
-		&List::send_notify_to_listmaster('automatic_list_creation_failed',$robot,["Failed to process $file : family $dyn_list_family does not exist, impossible to create the dynamic list."]);
+		&List::send_notify_to_listmaster(
+		    'automatic_list_creation_failed',
+		    $robot,
+		    {   'family' => $dyn_list_family,
+			'robot' => $robot,
+			'msg_id' => $msg_id,
+		    }
+		);
+		# FIXME: send DSN.
 		&report::reject_report_msg('user','list_unknown',$sender,{'listname' => $listname,'message' => $message},$robot,$message->{'msg_as_string'},'');
 		return undef;
             }
@@ -1337,11 +1410,19 @@ sub DoFile {
 		$dyn_just_created = 1;
 	    }else {
 		&Log::do_log('err', 'Unable to create list %s. Message %s ignored.', $listname,'msg_id' => $hdr->get('Message-Id'));
-		&report::reject_report_msg('intern',"Message ignored because automatic list '$listname' creation could not be performed.",$sender,
-					   {'msg_id' => $hdr->get('Message-Id'),'message' => $message}, $dyn_family->{'robot'},$message->{'msg_as_string'});
-		&List::send_notify_to_listmaster('automatic_list_creation_failed',$dyn_family->{'robot'},["Failed to process $file"]);
+		&List::send_notify_to_listmaster(
+		    'automatic_list_creation_failed',
+		    $dyn_family->{'robot'},
+		    {   'listname' => $listname,
+			'family' => $dyn_list_family,
+			'robot' => $robot,
+			'msg_id' => $msg_id,
+		    }
+		);
+		# FIXME: send DSN.
+		&report::reject_report_msg('user','dyn_cant_create',$sender,{'listname' => $listname,'message' => $message},$robot,$message->{'msg_as_string'},'');
 		&Log::db_log({'robot' => $dyn_family->{'robot'},'list' => $listname,'action' => 'DoFile',
-			      'parameters' => $hdr->get('Message-Id').",$dyn_family->{'robot'}",'target_email' => '','msg_id' => hdr->get('Message-Id'),'status' => 'error','error_type' => 'internal','user_email' => $sender,'client' => $ip,'daemon' => $daemon_name});
+			      'parameters' => $hdr->get('Message-Id').",$dyn_family->{'robot'}",'target_email' => '','msg_id' => $hdr->get('Message-Id'),'status' => 'error','error_type' => 'internal','user_email' => $sender,'client' => $ip,'daemon' => $daemon_name});
 		return undef ;
             }
         }
@@ -1360,7 +1441,7 @@ sub DoFile {
 		my $conf_loop_prevention_regex;
 		$conf_loop_prevention_regex = $list->{'admin'}{'loop_prevention_regex'};
 		$conf_loop_prevention_regex ||= &Conf::get_robot_conf($robot, 'loop_prevention_regex');
-		if ($sender =~ /^($conf_loop_prevention_regex)(\@|$)/mio) {
+		if ($sender =~ /^($conf_loop_prevention_regex)(\@|$)/mi) {
 			&do_log('notice','Ignoring message which would cause a loop, sent by %s', $sender);
 			return undef;
 		}
@@ -1448,9 +1529,9 @@ sub DoFile {
 		if ($Conf{'automatic_list_removal'} =~ /if_empty/i) {
 		    $list->close(); $list->purge(); # verifier pour tt ce bloc si supprime bien tout
 		    # but what about list_of_lists ?
-		    if (exists $List::list_of_lists{$list->{'domain'}}{$list->{'name'}}) { # test  virer si ok
+		    if (exists $List::list_of_lists{$list->{'domain'}}{$list->{'name'}}) { # test à virer si ok
 		      delete $List::list_of_lists{$list->{'domain'}}{$list->{'name'}};
-		      &do_log('err', 'la liste a t trouve dans la list_of_lists',$listname, $dyn_list_family);
+		      &do_log('err', 'la liste a été trouvée dans la list_of_lists',$listname, $dyn_list_family);
 		    }
 		}
                 return undef;
@@ -1463,9 +1544,9 @@ sub DoFile {
 		if ($Conf{'automatic_list_removal'} =~ /if_empty/i) {
 		    $list->close(); $list->purge(); # verifier pour tt ce bloc si supprime bien tout
 		    # but what about list_of_lists ?
-		    if (exists $List::list_of_lists{$list->{'domain'}}{$list->{'name'}}) { # test  virer si ok
+		    if (exists $List::list_of_lists{$list->{'domain'}}{$list->{'name'}}) { # test à virer si ok
 		      delete $List::list_of_lists{$list->{'domain'}}{$list->{'name'}};
-		      &do_log('err', 'la liste a t trouve dans la list_of_lists',$listname, $dyn_list_family);
+		      &do_log('err', 'la liste a été trouvée dans la list_of_lists',$listname, $dyn_list_family);
 		    }
 		}
                 return undef;
@@ -1488,7 +1569,7 @@ sub DoFile {
     }elsif ($rcpt =~ /^listmaster(\@(\S+))?$/) {
 	$status = &DoForward('sympa', 'listmaster', $robot, $message);
 
-	## Mail adressed to the robot and mail 
+	## Mail addressed to the robot and mail 
 	## to <list>-subscribe or <list>-unsubscribe are commands
     }elsif (($rcpt =~ /^(sympa|$conf_email)(\@\S+)?$/i) || ($type =~ /^(subscribe|unsubscribe)$/o)) {
 	$status = &DoCommand($rcpt, $robot, $message);
@@ -1538,7 +1619,7 @@ sub DoFile {
 	    ## Sampling delay is over, reinit
 	    $loop_info{$sender}{'date_init'} = time;
 
-	    ## We apply Decrease factor if a loop occured
+	    ## We apply Decrease factor if a loop occurred
 	    $loop_info{$sender}{'count'} *=  &Conf::get_robot_conf($robot,'loop_command_decrease_factor');
 	}
 
@@ -1929,7 +2010,7 @@ sub DoMessage{
 	    my $apply_dkim_signature = 'off';
 	    $apply_dkim_signature = 'on' if &tools::is_in_array($list->{'admin'}{'dkim_signature_apply_on'},'any');
 	    $apply_dkim_signature = 'on' if (&tools::is_in_array($list->{'admin'}{'dkim_signature_apply_on'},'smime_authenticated_messages') && ($is_signed->{'body'}));
-	    $apply_dkim_signature = 'on' if &tools::is_in_array($list->{'admin'}{'dkim_signature_apply_on'},'dkim_authenticated_messages');
+	    $apply_dkim_signature = 'on' if (&tools::is_in_array($list->{'admin'}{'dkim_signature_apply_on'},'dkim_authenticated_messages')  && $message->{'dkim_pass'});
 
 	    my $numsmtp;
 	    eval { $numsmtp =$list->distribute_msg('message'=> $message,
@@ -2073,8 +2154,10 @@ sub DoCommand {
     
     my $sender = $message->{'sender'};
 
-    if ($msg->{'spam_status'} eq 'spam'){
-	&do_log('notice', "Message for robot %s@%s ignored, because tagged as spam (Message-id: %s)",$rcpt,$robot,$messageid);
+    if ($message->{'spam_status'} eq 'spam') {
+	do_log('notice',
+	    'Message for list %s@%s ignored, because tagged as spam (Message-id: %s)',
+	    $rcpt, $robot, $messageid);
 	return undef;
     }
 
