@@ -1,7 +1,12 @@
 #!/usr/bin/perl
+# $Id: sympa_wizard.pl.in 11272 2014-07-29 15:19:43Z sikeda $
 
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,8 +19,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 =head1 NAME
 
@@ -49,7 +53,7 @@ Display usage instructions
 
 =item Serge Aumont <sa@cru.fr>
 
-=item Olivier Salan <os@cru.fr>
+=item Olivier SalaE<252>n <os@cru.fr>
 
 =back
 
@@ -63,6 +67,12 @@ use Getopt::Long;
 use Pod::Usage;
 use Sympa::Constants;
 
+my $with_CPAN; # check if module "CPAN" installed.
+
+BEGIN {
+    $with_CPAN = eval { require CPAN; };
+}
+
 ## sympa configuration files
 my $wwsympa_conf = Sympa::Constants::WWSCONFIG;
 my $sympa_conf   = Sympa::Constants::CONFIG;
@@ -287,7 +297,6 @@ sub edit_configuration {
 }
 
 sub check_cpan {
-    require CPAN;
 
     ## assume required_version = 1.0 if not specified.
     my %cpan_modules = (
@@ -303,22 +312,38 @@ sub check_cpan {
 				      usage => 'CAS Single Sign-On client libraries. Required if you configure Sympa to delegate web authentication to a CAS server.',
 				     },
 			'CGI' => {
-				  required_version =>'3.35',
+				  required_version =>'3.51',
 				  package_name => 'CGI',
 				  mandatory => 1,
 				  usage => 'required to run Sympa web interface',
 				  },
+			# CGI::Cookie is included in CGI.
+			# CGI::Fast is included in CGI.
 			'Crypt::CipherSaber' => {
 						 required_version =>'0.50',
-						 package_name => 'CipherSaber',
-						 usage => 'this module provides reversible encryption of user passwords in the database.usefull only when updating from old version with password reversible encryption.',
+						 package_name => 'Crypt-CipherSaber',
+						 usage => 'this module provides reversible encryption of user passwords in the database.  Useful when updating from old version with password reversible encryption, or if secure session cookies in non-SSL environments are required.',
 						},
+			'Crypt::OpenSSL::X509' => {
+						   required_version => '1.800.1',
+						   package_name => 'Crypt-OpenSSL-X509',
+						   usage => 'required to extract user certificates for SSL clients and S/MIME messages.',
+						  },
+			'Data::Password' => {
+				      required_version =>'1.07',
+				      package_name => 'Data-Password',
+				      usage => 'Used for configureable hardening of passwords via the password_validation sympa.conf directive.',
+				     },
 			'DB_File' => {
 				      required_version =>'1.75',
-				      package_name => 'DB_FILE',
+				      package_name => 'DB_File',
 				      mandatory => 1,
 				      usage => ' used for maintaining snapshots of list members',
 				     },
+			'DBD::ODBC' => {
+					package_name => 'DBD-ODBC',
+					usage => 'ODBC database driver, required if you connect to a database via ODBC.',
+					},
 			'DBD::Oracle' => {
 					  required_version =>'0.90',
 					  package_name => 'DBD-Oracle',
@@ -344,7 +369,7 @@ sub check_cpan {
 			'DBD::mysql' => {
 					 required_version =>'4.008',
 					 prerequisites => 'mysql-devel and myslq-server. mysql should be running for make test to succeed',
-					 package_name => 'Msql-Mysql-modules',
+					 package_name => 'DBD-mysql',
 					 mandatory => 1,
 					 usage => 'Mysql database driver, required if you connect to a Mysql database.\nYou first need to install the Mysql server and have it started before installing the Perl DBD module.',
 					},
@@ -366,10 +391,9 @@ sub check_cpan {
 				     usage => 'module for character encoding processing',
 				    },
 			'FCGI' => {
-				   required_version =>'0.67',
+				   required_version => '0.67',
 				   package_name => 'FCGI',
-				   mandatory => 1,
-				   usage => 'WWSympa, Sympa\'s web interface can run as a FastCGI (ie: a persistent CGI). If you install this module, you will also need to install the associated mod_fastcgi for Apache.',
+				   'usage' => "WWSympa, Sympa's web interface can run as a FastCGI (i.e. a persistent CGI). If you install this module, you will also need to install the associated FastCGI frontend, e.g. mod_fcgid for Apache.",
 				  },
 			'File::Copy::Recursive' => {
 						    required_version =>'0.36',
@@ -407,18 +431,36 @@ sub check_cpan {
 					      package_name => 'IO-Socket-SSL',
 					      usage => 'required when including members of a remote list',
 					     },
+			# Net::SSLeay is included in IO-Socket-SSL.
+
+			# Locale::Messages 1.16 or earlier does not have its
+			# own $VERSION variable.  Check the version of
+			# Locale::TextDomain instead.
 			'Locale::TextDomain' => {
 						 package_name => 'libintl-perl',
 						 mandatory => 1,
 						 usage => 'internationalization functions',
 						},
 			'LWP' => {
-				  package_name => 'libwww-per',
+				  package_name => 'libwww-perl',
 				  mandatory => 1,
 				  usage => 'required when including members of a remote list',
 				 },
+			'Mail::Address' => {
+					    required_version => '1.70',
+					    package_name => 'MailTools',
+					    mandatory => 1,
+					    usage => 'used to parse or build mailboxes in message headers',
+					   },
+			# Mail::DKIM::Signer is included in Mail-DKIM.
+			# Mail::DKIM::TextWrap is included in Mail-DKIM.
+			'Mail::DKIM::Verifier' => {
+					 required_version => '0.39',
+					 package_name => 'Mail-DKIM',
+					 usage => 'required in order to use DKIM features (both for signature verification and signature insertion)',
+					},
 			'MHonArc::UTF8' => {
-					    required_version =>'2.6.0',
+					    required_version =>'2.6.18',
 					    package_name => 'MHonArc',
 					    mandatory => 1, 
 					    usage => 'mhonarc is used to build Sympa web archives',
@@ -430,13 +472,13 @@ sub check_cpan {
 					   usage => 'required to compute digest for password and emails',
 					  },
 			'MIME::Charset' => {
-					    required_version =>'1.006.2',
+					    required_version =>'1.010',
 					    package_name => 'MIME-Charset',
 					    mandatory => 1,
 					    usage => 'used to encode mail body using a different charset',
 					   },
 			'MIME::EncWords' => {
-					     required_version =>'1.010',
+					     required_version =>'1.014',
 					     package_name => 'MIME-EncWords',
 					     mandatory => 1,
 					     usage => 'required to decode/encode SMTP header fields without breaking character encoding', 
@@ -469,6 +511,12 @@ sub check_cpan {
 					package_name => 'libnet',
 					usage => 'this is required if you set \'list_check_smtp\' sympa.conf parameter, used to check existing aliases before mailing list creation.',
 				       },
+			'Net::DNS' => {
+					required_version =>'0.65', 
+					package_name => 'Net-DNS',
+					mandatory => 1,
+					usage => 'this is required if you set a value for \'dmarc_protection_mode\' which requires DNS verification.',
+				       },
 			'perl' => {
 				   required_version =>'5.008',
 				  },
@@ -512,10 +560,6 @@ sub check_cpan {
 					  mandatory => 1,
 					  usage => 'used to parse list configuration templates and instanciate list families',
 					 },
-			'Mail::DKIM' => { required_version =>'0.36',
-					  package_name => 'Mail-DKIM',
-					  usage => 'required in order to use DKIM features (both for signature verification and signature insertion)',
-				      } 
 		       );
 
     print "##########################################################################################
@@ -530,47 +574,6 @@ Strike return key to continue...
 ";
     my $rep = <STDIN>;
 
-    my %opt_CPAN = (
-		'AuthCAS' => 'AuthCAS',
-        'DBD::Pg' => 'DBD-Pg',
-        'DBD::Oracle' => 'DBD-Oracle',
-        'DBD::Sybase' => 'DBD-Sybase',
-        'DBD::SQLite' => 'DBD-SQLite',
-        'Net::LDAP' =>   'perl-ldap',
-        'CGI::Fast' => 'CGI',
-        'Net::SMTP' => 'libnet',
-        'IO::Socket::SSL' => 'IO-Socket-SSL',
-        'Net::SSLeay' => 'NET-SSLeay',
-        'Bundle::LWP' => 'LWP',
-        'SOAP::Lite' => 'SOAP-Lite',
-        'File::NFSLock' => 'File-NFSLock',
-        'File::Copy::Recursive' => 'File-Copy-Recursive',
-        'Crypt::CipherSaber' => 'CipherSaber',
-	'mail::DKIM' =>'Mail-DKIM',
-    );
-
-    my %opt_features = (
-	'AuthCAS' => 'CAS Single Sign-On client libraries. Required if you configure Sympa to delegate web authentication to a CAS server.',
-        'DBI' => 'a generic Database Driver, required by Sympa to access Subscriber information and User preferences. An additional Database Driver is required for each database type you wish to connect to.',
-        'DBD::mysql' => 'Mysql database driver, required if you connect to a Mysql database.\nYou first need to install the Mysql server and have it started before installing the Perl DBD module.',
-        'DBD::Pg' => 'PostgreSQL database driver, required if you connect to a PostgreSQL database.',
-        'DBD::Oracle' => 'Oracle database driver, required if you connect to a Oracle database.',
-        'DBD::Sybase' => 'Sybase database driver, required if you connect to a Sybase database.',
-        'DBD::SQLite' => 'SQLite database driver, required if you connect to a SQLite database.',
-        'Net::LDAP' =>   'required to query LDAP directories. Sympa can do LDAP-based authentication ; it can also build mailing lists with LDAP-extracted members.',
-        'CGI::Fast' => 'WWSympa, Sympa\'s web interface can run as a FastCGI (ie: a persistent CGI). If you install this module, you will also need to install the associated mod_fastcgi for Apache.',
-        'Crypt::CipherSaber' => 'this module provides reversible encryption of user passwords in the database.usefull only when updating from old version with password reversible encryption.',
-        'Archive::Zip ' => 'this module provides zip/unzip for archive and shared document download/upload',
-        'FCGI' => 'WSympa, Sympa\'s web interface can run as a FastCGI (ie: a persistent CGI). If you install this module, you will also need to install the associated mod_fastcgi for Apache.',
-        'Net::SMTP' => 'this is required if you set \'list_check_smtp\' sympa.conf parameter, used to check existing aliases before mailing list creation.',
-        'IO::Socket::SSL' => 'required by CAS (single sign-on) and the \'include_remote_sympa_list\' feature that includes members of a list on a remote server, using X509 authentication',
-        'Net::SSLeay' => 'required by the \'include_remote_sympa_list\' feature that includes members of a list on a remote server, using X509 authentication',
-        'Bundle::LWP' => 'required by the \'include_remote_sympa_list\' feature that includes members of a list on a remote server, using X509 authentication',
-        'SOAP::Lite' => 'required if you want to run the Sympa SOAP server that provides ML services via a "web service"',
-        'File::NFSLock' => 'required to perform NFS lock ; see also lock_method sympa.conf parameter',
-	'mail::DKIM' => 'requiered in order to use Sympa DKIM features, both for signature verification  and signature insertion. it should be used event if you do not want your server to add signature !',
-    );
-
     ### main:
     print "******* Check perl for SYMPA ********\n";
     ### REQ perl version
@@ -651,7 +654,7 @@ sub check_modules {
 	    next;
 	  } else {
 	    print "version is too old ($v < $rv).\n";
-	    print ">>>>>>> You must update \"$cpan_modules->{$mod}{package_name}\" to version \"$cpan_modules->{$mod}{required_version}}\" <<<<<<.\n";
+	    print ">>>>>>> You must update \"$cpan_modules->{$mod}{package_name}\" to version \"$cpan_modules->{$mod}{required_version}\" <<<<<<.\n";
 	    install_module($mod, {'default' => $default}, $cpan_modules);
 	  }
         }
@@ -662,8 +665,9 @@ sub check_modules {
 # Install a CPAN module
 ##----------------------
 sub install_module {
-#    my ($module, $options, $opt_features) = @_;
-  my ($module, $options, $cpan_modules) = @_;
+    return unless $with_CPAN;
+
+    my ($module, $options, $cpan_modules) = @_;
 
     my $default = $options->{'default'};
 
