@@ -1,11 +1,14 @@
 #! /usr/bin/perl
 
 # task_manager.pl - This script runs as a daemon and processes periodical Sympa tasks
-# RCS Identication ; $Revision: 7451 $ ; $Date: 2012-07-06 11:21:24 +0200 (ven 06 jui 2012) $ 
+# RCS Identication ; $Revision: 10713 $ ; $Date: 2014-05-23 11:18:40 +0200 (ven. 23 mai 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +21,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ## Options :  F         -> do not detach TTY
 ##         :  d		-> debug -d is equiv to -dF
@@ -115,7 +117,7 @@ unless ($main::options{'debug'} || $main::options{'foreground'}) {
 ## If process is running in foreground, don't write STDERR to a dedicated file
 my $options;
 $options->{'stderr_to_tty'} = 1 if ($main::options{'foreground'});
-&tools::write_pid($wwsconf->{'task_manager_pidfile'}, $$, $options);
+tools::write_pid('task_manager', $$, $options);
 
 $wwsconf->{'log_facility'}||= $Conf{'syslog'};
 do_openlog($wwsconf->{'log_facility'}, $Conf{'log_socket_type'}, 'task_manager');
@@ -390,7 +392,7 @@ while (!$end) {
 }
 
 &do_log ('notice', 'task_manager exited normally due to signal'); 
-&tools::remove_pid($wwsconf->{'task_manager_pidfile'}, $$);
+tools::remove_pid('task_manager', $$);
 
 exit(0);
 
@@ -1152,8 +1154,9 @@ sub purge_user_table {
     }
 
     foreach my $r (keys %{$Conf{'robots'}}) {
-
-	my $all_lists = &List::get_lists($r);
+	## To prevent information loss by misconfiguration, we won't use
+	## list_table cache!
+	my $all_lists = &List::get_lists($r, { 'use_files' => 1 });
 	foreach my $list (@$all_lists){
 
 	    ## Owners
@@ -1745,7 +1748,7 @@ sub error {
     my $message = $_[1];
 
     my @param;
-    $param[0] = "An error has occured during the execution of the task $task_file :
+    $param[0] = "An error has occurred during the execution of the task $task_file :
                  $message";
     do_log ('err', "$message");
     change_label ($task_file, 'ERROR') unless ($task_file eq '');
